package OOTPOU::ExportTracker;

use strict;
use vars qw(@ISA);
use OOTPOU::Page;
use OOTPOU::Utils;
use OOTPOU::HTMLUtils;
use File::stat;
use Time::localtime;

@ISA = "OOTPOU::Page";

sub printPage {

    my ($self) = @_;

    my $info;
    my %teams;
    my %subLeagues;
    my %divisions;
    my $subleague;
    my $division;
    my $team;
    my $count;
    my $value;
    my $lgDate;
    my $dispDate;
    my $new;

    open(IN, "$self->{INFO}->{CSV}/teams.csv") or die("Cannot open $self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
	if (/,0,"[^,]*?"\s*$/ && /^(\d+),"(.*?)","\w+","(.*?)",".*?","(.*?)",\d+,\d+,$self->{INFO}->{LEAGUEID},(\d+),/) {
	    $teams{$1} = {};
	    $teams{$1}->{CITY} = $2;
	    $teams{$1}->{NAME} = $3;
	    $teams{$1}->{IMAGE} = $4;
	    $teams{$1}->{SUBLEAGUE} = $5;
	}
    }
    close(IN);
    
    open(IN, "$self->{INFO}->{CSV}/sub_leagues.csv") or die("Cannot open $self->{INFO}->{CSV}/sub_leagues.csv");
    while (<IN>) {
	if (/^$self->{INFO}->{LEAGUEID},(\d+),"(.*?)",/) {
	    $subLeagues{$1} = {};
	    $subLeagues{$1}->{NAME} = $2;
	}
    }
    close(IN);
    
    open(IN, "$self->{INFO}->{CSV}/divisions.csv") or die("Cannot open $self->{INFO}->{CSV}/divisions.csv");
    while (<IN>) {
	if (/^$self->{INFO}->{LEAGUEID},(\d+),(\d+),"(.*?)",/) {
	    $divisions{"$1-$2"} = {};
	    $divisions{"$1-$2"}->{NAME} = $3;
	}
    }
    close (IN);
    
    open(IN, "$self->{INFO}->{CSV}/team_relations.csv") or die("Cannot open $self->{INFO}->{CSV}/team_relations.csv");
    while (<IN>) {
	if (/^$self->{INFO}->{LEAGUEID},(\d+),(\d+),(\d+)\s*$/) {
	    $teams{$3}->{DIVISION} = "$1-$2";
	}
    }
    close (IN);
    
    ($lgDate, $dispDate) = $self->getLgFileTime();
    print '<table cellspacing="0" cellpadding="1" class="lpo" width="100%" style="margin-bottom:12px;">';
    print "<tr><td class=\"h1_l\">League file last updated: $dispDate</td></tr>\n";
	  print "</table>\n";
	  
    print '<table cellspacing="0" cellpadding="1" class="lpo" width="100%" style="margin-bottom:12px;">';
    foreach $subleague (sort {$a <=> $b} (keys %subLeagues)) {
	print "<tr><td class=\"h1_l\">$subLeagues{$subleague}->{NAME}</td></tr>\n";
	print '<tr><td style="padding-top:12px;padding-left:12px;padding-right:12px;">';
	print '<table cellspacing="0" cellpadding="1" class="lpo" width="840" style="margin-bottom:12px;">';
	foreach $division (sort {$a cmp $b} (keys %divisions)) {
	    if ($division =~ /^$subleague-/) {
		print "<tr><td class=\"h1_l\" colspan=\"3\">$divisions{$division}->{NAME}</td></tr>\n";
		print '<tr><td align="center" style="padding-top:0px;padding-bottom:2px;"><table cellspacing="0" cellpadding="1" width="300">';
		$count = 0;
		foreach $team (keys %teams) {
		    if ($teams{$team}->{DIVISION} eq $division) {
			if ($count == 0) {
			    print "<tr>\n";
			}
			print "<td width=\"50\"><img src=\"$self->{INFO}->{URL}/$teams{$team}->{IMAGE}\" width=\"40\" height=\"40\"></td><td width=\"249\"><table cellspacing=\"0\" cellpadding=\"1\" width=\"249\"><tr><td><span style=\"font-weight:bold;\">$teams{$team}->{CITY} $teams{$team}->{NAME}</span></td></tr>\n";
			print "<tr><td>";
			($value, $new) = $self->getExportTime($team, $lgDate);
			if ($value eq 'NONE') {
			    print 'No Export';
			}
			else {
			    if ($new == 1) {
				print '<font color="#000088">';
			    }
			    else {
				print '<font color="#880000">';
			    }
			    print "Last Export: $value</font>";
			}
			print '</td></tr></table></td>';
			if ($count == 2) {
			    print "</tr>";
			}
			$count++;
			if ($count > 2) {
			    $count = 0;
			}
		    }
		}
		print "</table></td></tr>\n";
	    }
	}
	print "</table></td></tr>\n";
    }
}

sub getExportTime {
    my ($self, $team, $lgDate) = @_;
    my $datetime_string;
    my $file = "$self->{INFO}->{EXPORT_DIR}/team_$team.ootp";
    #my $output = `ls -l $self->{INFO}->{EXPORT_DIR}/team_$team.ootp`;
    my $new = 0;
    if (-e $file) {
      $datetime_string = stat($file)->mtime;
      if ($lgDate < $datetime_string) {
        $new = 1;
      }
      return (ctime($datetime_string), $new);
    }
    else {
      return ("NONE", $new);
    }
    
    #if (!$output) {
	  # $new = 0;
    # my @output2 = `ls -ltr $self->{INFO}->{EXPORT_DIR}/team_$team.ootp.imported*`;
	  # foreach my $output2 (@output2) {
	  #  $output = $output2;
	  # }
    #}
    #if (!$output) {
    #  return ('NONE', $new);
    #}
    #if ($output =~ /(\w+)\s*(\d+)\s*(\d+:\d+)\s*/) {
    #  return ("$1 $2, $3", $new);
    #}
    #else {
    #  return ('NA', $new);
    #}
}

sub getLgFileTime {
    my ($self) = @_;
    my $datetime_string;
    my $file = "$self->{INFO}->{EXPORT_DIR}/$self->{INFO}->{LEAGUE_NAME}.lg.tar.gz";
    if (-e $file) {
      $datetime_string = stat($file)->mtime;
      return($datetime_string, ctime($datetime_string));
    }
    else {
      	return(0, "No file uploaded");
    }
    
}
1
