package OOTPOU::LSAdmin;

use strict;
use vars qw(@ISA);
use OOTPOU::Page;
use OOTPOU::Utils;
use OOTPOU::HTMLUtils;

@ISA = "OOTPOU::Page";

sub printPage {
    my ($self) = @_;

    if (!$self->{INFO}->{ISCOMMISH}) {
        return;
    }

    $self->getInfo();

    if ($self->{PARAMS}->{genReport}) {
	my $leagues = $self->OOTPOU::Utils::getActiveLeagues;
	my $league;
	foreach $league (keys %{$leagues}) {
	    $self->genReport($league);
	    $self->simSeasons($league);
	}
    }
    elsif ($self->{PARAMS}->{save}) {
	$self->saveInfo();
    }

    my $date = OOTPOU::Utils::getDate($self->{INFO}->{CSV}, $self->{INFO}->{LEAGUEID});
    my $currTime = localtime;

    print "<tr><td>\n";
    print "<form>\n";
    print "<table><tr><td>\n";
    print "<table class=\"lpo\" width=500>";
    print '<tr><td colspan="2" class="h2_l">Parameters</td></tr>';
    print '<tr><td colspan=2><table width="100%"><tr><td>Sim Length (in days)</td>';
    print "<td align=\"left\"><input type=\"text\" name=\"SIMLENGTH\" size=3 value=\"$self->{SIMLENGTH}\"/></td></tr>";
    print '<tr><td colspan="2" class="h2_l">Playoff Odds Configuration</td></tr>';
    print '<tr><td>';
    print "Postseason Qualifiers:</td><td><select name=\"PSFORMAT\">\n";
    print "<option value=\"1\"";
    if ($self->{PSFORMAT} == 1) {print " selected"};
    print ">Division Winners</option>\n<option value=\"2\"";
    if ($self->{PSFORMAT} == 2) {print " selected"};
    print ">Division Winners + Wild Card(s)</option>\n</select></td></tr><tr><td align=\"left\">\n";
    print "Estimate Future Success Based On:</td><td><select name=\"WINPCT\">\n";
    print "<option value=\"1\"";
    if ($self->{WINPCT} == 1) {print " selected"};
    print ">Pythagorean Win Pct (Exp)</option>\n";
    print "<option value=\"2\"";
    if ($self->{WINPCT} == 2) {print " selected"};
    print ">Pythagorean Win Pct (Pythagenpat)</option>\n";
    print "<option value=\"3\"";
    if ($self->{WINPCT} == 3) {print " selected"};
    print ">Actual Win Pct</option>\n</select></td></tr>";
    print "<tr><td colspan align=\"left\">Pythagorean Exponent:</td><td><input type=\"text\" name=\"PYTHEXP\" size=5 value=\"$self->{PYTHEXP}\"/></td></tr>";
    print "<tr><td colspan align=\"left\">Seasons to Simulate:</td><td><input type=\"text\" name=\"SEASONS\" size=5 value=\"$self->{SEASONS}\"/></td></tr>";

    print '</table></td></tr><tr><td></td><td align="right"><input type="submit" name="save" value="Save"/></td></tr></table>';
    print '</td><td width="100%" align="right" valign="top">';
    print "<table cellspacing=\"0\" cellpadding=\"2\" class=\"lpo\" width=\"100%\">";
    print "<tr><td class=\"s1_l\">Last Update of Sim Report</td><td class=\"s1_l\">$self->{LS_TIME}</td></tr>\n";
    print "<tr><td class=\"s2_l\">Current Time</td><td class=\"s2_l\">$currTime</td></tr>\n";
    print "<tr><td class=\"s1_l\">League Date of Sim Reports</td><td class=\"s1_l\">$self->{LS_DATE}</td></tr>\n";
    print "<tr><td class=\"s2_l\">Current Date in CSV Files</td><td class=\"s2_l\">$date</td></tr>\n";
    print "</table></td></tr></table>";
    print "</td></tr>";
    print "<tr><td colspan=2><br/><table><tr><td>\n";
    print "<input type='submit' name='genReport' value='Generate Report'/></td><td>\n";
    print " - Generate the report for the latest sim.</td></tr></table>\n";
    print "<input type=\"hidden\" name=\"page\" value=\"lsAdmin\"/></form></td></tr>\n";
}

sub genReport {
    my ($self, $leagueid) = @_;

    my %players;
    my %games;
    my %teams;
    my $id;

    if (!-e "OOTPOU/lastSim") {
	`mkdir OOTPOU/lastSim`;
    }
    if (!-e "OOTPOU/lastSim/$leagueid") {
	`mkdir OOTPOU/lastSim/$leagueid`;
    }

    my $output = `grep '^$leagueid,' $self->{INFO}->{CSV}/leagues.csv`;
    my $openMonth;
    my $openDay;
    if ($output =~ /^$leagueid,".*?",".*?",\d+,\d+,\d+,\d+,".*?",".*?","\d+-(\d+)-(\d+)",/) {
	$openMonth = $1;
	$openDay = $2;
    }

    open (IN, "$self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/^(\d+),"(.*?)","(\w+)","(.*?)","(.*?)",".*?",\d+,\d+,$leagueid,/) {
	    $id = $1;
            $teams{$1} = {};
            $teams{$1}->{CITY}    = $2;
            $teams{$1}->{ABBREV}  = $3;
            $teams{$1}->{NAME}    = $4;
            $teams{$1}->{LOGO}    = $5;
            $teams{$1}->{GAMES}   = [];
            $teams{$1}->{PLAYERS} = {};
	    $teams{$1}->{AWINS}   = 0;
	    $teams{$1}->{ALOSS}   = 0;
	    $teams{$1}->{HWINS}   = 0;
            $teams{$1}->{HLOSS}   = 0;
	    $teams{$1}->{SIMW}    = 0;
	    $teams{$1}->{SIML}    = 0;

	    $teams{$1}->{AB}  = 0;
	    $teams{$1}->{PA}  = 0;
	    $teams{$1}->{G}   = 0;
	    $teams{$1}->{H}   = 0;
	    $teams{$1}->{HR}  = 0;
	    $teams{$1}->{B2}  = 0;
	    $teams{$1}->{B3}  = 0;
	    $teams{$1}->{R}   = 0;
	    $teams{$1}->{RBI} = 0;
	    $teams{$1}->{K}   = 0;
	    $teams{$1}->{BB}  = 0;
	    $teams{$1}->{SB}  = 0;
	    $teams{$1}->{CS}  = 0;
	    $teams{$1}->{HBP} = 0;
	    $teams{$1}->{SF}  = 0;

	    $teams{$1}->{IP}  = 0;
	    $teams{$1}->{IPF} = 0;
	    $teams{$1}->{PG}  = 0;
	    $teams{$1}->{PGS} = 0;
	    $teams{$1}->{PAB} = 0;
	    $teams{$1}->{PTB} = 0;
	    $teams{$1}->{PH}  = 0;
	    $teams{$1}->{PBB} = 0;
	    $teams{$1}->{PK}  = 0;
	    $teams{$1}->{PR}  = 0;
	    $teams{$1}->{PER} = 0;
	    $teams{$1}->{W}   = 0;
	    $teams{$1}->{L}   = 0;
	    $teams{$1}->{S}   = 0;
	    $teams{$1}->{CG}  = 0;
	    $teams{$1}->{SO}  = 0;
	    $teams{$1}->{PHR} = 0;
	    $teams{$1}->{PSF} = 0;
	    /,(\d+),"[^,]*?"\s*$/;
	    $teams{$id}->{ALLSTAR} = $1;
        }
    }
    close (IN);

    my $value;
    my $month = $self->{INFO}->{DATE}->{MONTH};
    my $year  = $self->{INFO}->{DATE}->{YEAR};
    my $day   = $self->{INFO}->{DATE}->{DAY};

    my ($minDate, $maxDate) = $self->getDateRange();
    open (IN, "$self->{INFO}->{CSV}/games.csv");
    while (<IN>) {
        if (/^(\d+),$leagueid,(\d+),(\d+),(\d+),"($year)-(\d+)-(\d+)",(\d+),(\d+),(\d+),\d+,(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)/) {
	    if ($10 && $9 == 0) {
		if ($12 > $13) {
		    $teams{$3}->{AWINS}++;
		    $teams{$2}->{HLOSS}++;
		}
		else {
                    $teams{$3}->{ALOSS}++;
                    $teams{$2}->{HWINS}++;
		}
	    }
            if ((($6*100)+$7 >= $minDate) && (($6*100)+$7 <= $maxDate)) {
                $games{$1} = {};
                $games{$1}->{HTEAM}   = $2;
                $games{$1}->{ATEAM}   = $3;
                $games{$1}->{ATT}     = $4;
                $games{$1}->{YEAR}    = $5;
                $games{$1}->{MONTH}   = $6;
                $games{$1}->{DAY}     = $7;
                $games{$1}->{TIME}    = $8;
                $games{$1}->{TYPE}    = $9;
		$games{$1}->{SIMMED}  = $10;
		# TODO: Remove:
#		if (($6*100)+$7 >= (100*$month)+$day) {
#		    $games{$1}->{SIMMED} = 0;
#		}
                $games{$1}->{INNINGS} = $11;
                $games{$1}->{ARUNS}   = $12;
                $games{$1}->{HRUNS}   = $13;
                $games{$1}->{AHITS}   = $14;
                $games{$1}->{HHITS}   = $15;
                $games{$1}->{AERRS}   = $16;
                $games{$1}->{HERRS}   = $17;
                $games{$1}->{WP}      = $18;
                $games{$1}->{LP}      = $19;
                $games{$1}->{SP}      = $20;
		if ($9 != 4) {
		    push @{$teams{$2}->{GAMES}}, $1;
		    push @{$teams{$3}->{GAMES}}, $1;
		    $games{$1}->{SCORE} = 0;
		}
		else {
		    $games{$1}->{SCORE} = 100;
		}
                $games{$1}->{HR}   = {};
                $games{$1}->{SCR0} = {};
                $games{$1}->{SCR1} = {};
		
		if ($games{$1}->{SIMMED}) {
		    if ($9 != 4) {
			if ($games{$1}->{ARUNS} > $games{$1}->{HRUNS}) {
			    $teams{$games{$1}->{HTEAM}}->{SIML}++;
			    $teams{$games{$1}->{ATEAM}}->{SIMW}++;
			}
			else {
			    $teams{$games{$1}->{ATEAM}}->{SIML}++;
			    $teams{$games{$1}->{HTEAM}}->{SIMW}++;
			}
		    }
		    if ($games{$1}->{ARUNS} == 0 || $games{$1}->{HRUNS} == 0)
		    {
			$games{$1}->{SCORE}++;
		    }
		    if ($games{$1}->{ARUNS} + $games{$1}->{HRUNS} == 1)
		    {
			$games{$1}->{SCORE} += 2;
		    }
		    if ($games{$1}->{ARUNS} + $games{$1}->{HRUNS} < 4)
		    {
			$games{$1}->{SCORE} ++;
		    }
		    if ($games{$1}->{AHITS} == 0 || $games{$1}->{HHITS} == 0)
		    {
			$games{$1}->{SCORE} += 20;
		    }
		    if ($games{$1}->{AHITS} < 4)
		    {
			$games{$1}->{SCORE} += 4 - $games{$1}->{AHITS};
		    }
		    if ($games{$1}->{HHITS} < 4)
		    {
			$games{$1}->{SCORE} += 4 - $games{$1}->{HHITS};
		    }
		    if ($games{$1}->{AHITS} >= 20 || $games{$1}->{HHITS} >= 20)
		    {
			$games{$1}->{SCORE} += 2;
		    }
		    if ($games{$1}->{ARUNS} >= 10 && $games{$1}->{HRUNS} >= 10)
		    {
			$games{$1}->{SCORE} ++;
		    }
		    if ($games{$1}->{ARUNS} >= 15 && $games{$1}->{HRUNS} >= 15)
		    {
			$games{$1}->{SCORE} += 2;
		    }
		    if ($games{$1}->{ARUNS} >= 20 || $games{$1}->{HRUNS} >= 20)
		    {
			$games{$1}->{SCORE} += 4;
		    }
		    if ($games{$1}->{INNINGS} > 10)
		    {
			$games{$1}->{SCORE}++;
		    }
		    if ($games{$1}->{INNINGS} > 12)
		    {
			$games{$1}->{SCORE}++;
		    }
		    if ($games{$1}->{INNINGS} > 14)
		    {
			$games{$1}->{SCORE} += 2;
		    }
		    if ($games{$1}->{INNINGS} > 15)
		    {
			$games{$1}->{SCORE} += 5;
		    }
                }
            }
        }
    }
    close(IN);

    open (IN, "$self->{INFO}->{CSV}/players.csv");
    while (<IN>) {
        if (/^(\d+),(\d+),\d+,"(.*?)","(.*?)",".*?",\d+,"\d+-\d+-\d+",\d+,\d+,\d+,\d+,\d+,0,\d+,\d+,\d+,(\d+),/) {
            if ($teams{$5}) {
                $players{$1} = {};
                $players{$1}->{ORG} = $5;
                $players{$1}->{TEAM} = $2;
                $players{$1}->{FNAME} = $3;
                $players{$1}->{LNAME} = $4;
		$players{$1}->{SCORE} = 0;
		$players{$1}->{PSCORE} = 0;
                if ($2 != $5) {
                    $players{$1}->{DEVONLY} = 1;
                }
                else {
                    $players{$1}->{AB}  = 0;
                    $players{$1}->{PA}  = 0;
                    $players{$1}->{G}   = 0;
                    $players{$1}->{H}   = 0;
                    $players{$1}->{HR}  = 0;
                    $players{$1}->{B2}  = 0;
                    $players{$1}->{B3}  = 0;
                    $players{$1}->{R}   = 0;
                    $players{$1}->{RBI} = 0;
                    $players{$1}->{K}   = 0;
                    $players{$1}->{BB}  = 0;
                    $players{$1}->{SB}  = 0;
                    $players{$1}->{CS}  = 0;
                    $players{$1}->{HBP} = 0;
                    $players{$1}->{SF}  = 0;

                    $players{$1}->{IP}  = 0;
                    $players{$1}->{IPF} = 0;
		    $players{$1}->{BF}  = 0;
                    $players{$1}->{PG}  = 0;
                    $players{$1}->{PGS} = 0;
                    $players{$1}->{PAB} = 0;
                    $players{$1}->{PTB} = 0;
                    $players{$1}->{PBB} = 0;
                    $players{$1}->{PH}  = 0;
                    $players{$1}->{PBB} = 0;
                    $players{$1}->{PK}  = 0;
                    $players{$1}->{PR}  = 0;
                    $players{$1}->{PHR} = 0;
                    $players{$1}->{PSF} = 0;
                    $players{$1}->{PER} = 0;
                    $players{$1}->{W}   = 0;
                    $players{$1}->{L}   = 0;
                    $players{$1}->{S}   = 0;
                    $players{$1}->{CG}  = 0;
                    $players{$1}->{SO}  = 0;

		    $teams{$2}->{PLAYERS}->{$1} = 1;
                }
            }
        }
    }
    close (IN);

    my $score;
    my $inn;
    open (IN, "$self->{INFO}->{CSV}/games_score.csv");
    while (<IN>) {
        if (/^(\d+),(\d+),(\d+),(\d+)\s*$/) {
            if ($games{$1}) {
                $score = "SCR$2";
                $games{$1}->{$score}->{$3} = $4;
            }
        }
    }
    close (IN);

    my $i;
    foreach $value (keys %games)
    {
	if ($games{$value}->{SIMMED} == 0) {
	    next;
	}

        $inn = $games{$value}->{INNINGS};
        $score = $games{$value}->{HRUNS} - $games{$value}->{ARUNS};
        if ($score == -1 || $score == 1)
        {
            $games{$value}->{SCORE} ++;
        }
        if ($score < 0 && $score + $games{$value}->{SCR0}->{$inn}+1 > 0)
        {
            $games{$value}->{SCORE} += $score + $games{$value}->{SCR0}->{$inn}+1;
        }
        elsif ($score > 0 && $games{$value}->{SCR1}->{$inn} - $score + 1 > 0)
        {
            $games{$value}->{SCORE} += $games{$value}->{SCR1}->{$inn} - $score + 1;
        }

        if ($inn > 9)
        {
            $i = 9;
            $games{$value}->{SCORE} += $games{$value}->{SCR0}->{9};
            if ($score > 0)
            {
                $games{$value}->{SCORE} += $games{$value}->{SCR0}->{$inn};
            }
            while ($i < $inn-1)
            {
                $games{$value}->{SCORE} += $games{$value}->{SCR1}->{$i};
                $i++;
            }
        }
    }


    open (IN, "$self->{INFO}->{CSV}/players_game_batting.csv");
    while (<IN>) {
        if (/^\d+,$year,\d+,(\d+),/) {
            if ($games{$1} && $games{$1}->{SIMMED}) {
                if (/^(\d+),$year,\d+,(\d+),\d+,\d+,\d+,\d+,(\d+),(\d+),(\d+),(\d+),\d+,(\d+),\d+,(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),\d+,\d+,\d+,(\d+),(\d+),/) {
                    $players{$1}->{AB}  += $3;
                    $players{$1}->{H}   += $4;
                    $players{$1}->{K}   += $5;
                    $players{$1}->{PA}  += $6;
                    $players{$1}->{G}   += $7;
                    $players{$1}->{B2}  += $8;
                    $players{$1}->{B3}  += $9;
                    $players{$1}->{HR}  += $10;
                    $players{$1}->{R}   += $11;
                    $players{$1}->{RBI} += $12;
                    $players{$1}->{SB}  += $13;
                    $players{$1}->{CS}  += $14;
                    $players{$1}->{BB}  += $15;
                    $players{$1}->{SF}  += $16;
                    $players{$1}->{HBP} += $17;

                    if ($10 > 0) {
                        $games{$2}->{HR}->{$1} = $10;
                    }
                }
            }
        }
    }
    close (IN);

    open (IN, "$self->{INFO}->{CSV}/players_game_pitching_stats.csv");
    while (<IN>) {
        if (/^\d+,$year,\d+,(\d+),/) {
            if ($games{$1} && $games{$1}->{SIMMED}) {
                if (/^(\d+),$year,\d+,(\d+),\d+,\d+,\d+,(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),\d+,(\d+),(\d+),(\d+),\d+,\d+,\d+,(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),\d+,\d+,\d+,(\d+),\d+,(\d+),/) {
                    $i = $1;
                    $players{$1}->{IP}   += $3;
                    $players{$1}->{PAB}  += $4;
                    $players{$1}->{PTB}  += $5;
                    $players{$1}->{PH}   += $6;
                    $players{$1}->{PK}   += $7;
		    $players{$1}->{BF}   += $8;
		    $players{$1}->{PBB}  += $9;
                    $players{$1}->{PR}   += $10;
                    $players{$1}->{PER}  += $11;
                    $players{$1}->{IPF}  += $12;
                    $players{$1}->{PG}   += $13;
		    $players{$1}->{PGS}  += $14;
                    $players{$1}->{W}    += $15;
                    $players{$1}->{L}    += $16;
                    $players{$1}->{S}    += $17;
                    $players{$1}->{PSF}  += $18;
                    $players{$1}->{PHR}  += $19;

                    if (/,(\d+),(\d+),\d+,\d+,\d+,\d+\.\d+\s*$/) {
                        $players{$i}->{CG}  += $1;
                        $players{$i}->{SO}  += $2;
                    }
                }
            }
        }
    }
    close (IN);

    open (IN, "<$self->{INFO}->{CSV}/players_career_batting_stats.csv");
    while (<IN>) {
	if (/^(\d+),$self->{INFO}->{DATE}->{YEAR},\d+,\d+,$leagueid,1,1,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),/) {
	    if ($players{$1}) {
		$players{$1}->{TOTHR} = $2;
	    }
	}
    }
    close (IN);

    open (IN, "<$self->{INFO}->{CSV}/players_career_pitching_stats.csv");
    while (<IN>) {
	if (/^(\d+),$self->{INFO}->{DATE}->{YEAR},\d+,\d+,$leagueid,1,1,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),(\d+),(\d+),/)
        {
	    if ($players{$1}) {
		$players{$1}->{TOTW} = $2;
		$players{$1}->{TOTL} = $3;
		$players{$1}->{TOTS} = $4;
	    }
	}
    }
    close (IN);

    my @keys = keys %teams;
    my @teamKeys = sort {$teams{$a}->{CITY} cmp $teams{$b}->{CITY}} @keys;
    my $team;

    my $teamChooser = "<SCRIPT language=\"Javascript\">\n";
    $teamChooser .= "function setTeam(team) {\n";
    $teamChooser .= "document.chooseTeam.team.value=team;\n";
    $teamChooser .= "}\n";
    $teamChooser .= "</SCRIPT>\n";
    $teamChooser .= "<tr><td><form name=\"chooseTeam\" method=\"get\">\n";
    $teamChooser .= "<input type=\"hidden\" name=\"page\" value=\"XXXPAGEXXX\"/>\n";
    $teamChooser .= "Select a Team: <select name=\"team\">\n";
    foreach $team (@teamKeys) {
	if (!$teams{$team}->{ALLSTAR}) {
	    $teamChooser .= "<option value=\"$team\">$teams{$team}->{CITY} $teams{$team}->{NAME}</option>\n";
	}
    }
    $teamChooser .= "</select>\n";
    $teamChooser .= "</select>&nbsp;<input type=\"submit\" value=\"Go\"></input></form></td></tr>\n";

    open (OUT, ">OOTPOU/lastSim/$leagueid/t0res.html");
    print OUT $teamChooser;
    close (OUT);

    open (OUT, ">OOTPOU/lastSim/$leagueid/t0stats.html");
    print OUT $teamChooser;
    close (OUT);

    my $gCount;
    my $j;
    my $game;
    my $hTeam;
    my $aTeam;
    my $player;
    my $totInn;
    my $aHR;
    my $hHR;
    my $key;
    my $c;
    my $tb;
    my $avg;
    my $obp;
    my $slg;
    my $ops;

    foreach $team (@teamKeys) {
        open (OUT, ">OOTPOU/lastSim/$leagueid/t".$team."res.html");

        print OUT $teamChooser;
        print OUT '<tr><td colspan="2"><table>';
        print OUT '<tr><td valign="top"><table class="lpo" width="460">';
        print OUT "<tr><td class=\"h1_l\"> Last Sim's Results ($teams{$team}->{SIMW}-$teams{$team}->{SIML})</td></tr>";
	print OUT "<tr><td colspan=\"2\"><img src=\"$self->{INFO}->{URL}/images/pixel.gif\" width=\"1\" height=\"12\"></td></tr>\n";

	@{$teams{$team}->{GAMES}} = sort {$games{$a}->{MONTH}*100+$games{$a}->{DAY} <=> $games{$b}->{MONTH}*100+$games{$b}->{DAY}} @{$teams{$team}->{GAMES}};

        $gCount = 0;
        while ($gCount < $self->{SIMLENGTH}) {
            $game = ${@{$teams{$team}->{GAMES}}}[$gCount];
	    if (!$game || $games{$game}->{MONTH} > $month || ($games{$game}->{MONTH} == $month && $games{$game}->{DAY} >= $day)) {
		last;
	    }

            $hTeam = $games{$game}->{HTEAM};
            $aTeam = $games{$game}->{ATEAM};
            print OUT '<tr><td align="left" valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:4px;">';
            print OUT OOTPOU::Utils::textDate("$games{$game}->{YEAR}-$games{$game}->{MONTH}-$games{$game}->{DAY}").': ';
            print OUT "<a href=\"$self->{INFO}->{URL}/box_scores/game_box_$game.html\">Box Scores</a> | ";
            print OUT "<a href=\"$self->{INFO}->{URL}/game_logs/log_$game.html\">Game Log</a> </td>\n";
#            print OUT '<td align="left" valign="top" style="padding-top:0px;padding-left:6px;padding-right:12px;padding-bottom:4px;"> </td></tr>';
            print OUT '<tr><td valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:12px;">';
            print OUT '<table cellspacing="0" cellpadding="0" width="423" class="lpo">';
            print OUT '<tr><td valign="top" width="44" style="padding-top:1px;padding-left:1px;">';
            print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$aTeam}->{LOGO}\" width=\"40\" height=\"40\">";
            print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$hTeam}->{LOGO}\" width=\"40\" height=\"40\"></td>\n";
            print OUT '<td width="405"><table cellspacing="0" cellpadding="1" width="380" style="border-left-width:1px; border-left-style:solid; border-left-color:#999999;">';
            print OUT '<tr><td class="hc">&#160;</td>';
#            print OUT "<tr><td class=\"hc\">$games{$game}->{SCORE}</td>\n";

            $inn = $games{$game}->{INNINGS};
            $totInn = $inn;
            if ($totInn > 10) {
                $i = $totInn-10;
            }
            else {
                $totInn = 10;
                $i = 1;
            }
            $j = $i;
            while ($i <= $totInn) {
                print OUT "<td class=\"hc\" width=\"18\">$i</td>\n";
                $i++;
            }
            print OUT '<td class="hc" width="18">R</td><td class="hc" width="18">H</td><td class="hc" width="18">E</td></tr>';
            print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
            print OUT "<a href=\"$self->{INFO}->{URL}/teams/team_$aTeam.html\">$teams{$aTeam}->{CITY}</a></td>\n";
            $i = $j;
            while ($i <= $totInn) {
                if ($i > $inn) {
                    $value = '&#160;';
                }
                else {
                    $value = $games{$game}->{SCR0}->{$i};
                }
                print OUT "<td class=\"ic\" width=\"18\">$value</td>";
                $i++;
            }
            print OUT "<td class=\"icgb\" width=\"18\">$games{$game}->{ARUNS}</td>\n";
            print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{AHITS}</td>\n";
            print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{AERRS}</td></tr>\n";
            print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
            print OUT "<a href=\"$self->{INFO}->{URL}/teams/team_$hTeam.html\">$teams{$hTeam}->{CITY}</a></td>\n";
            $i = $j;
            while ($i <= $totInn) {
                if ($i > $inn) {
                    $value = '&#160;';
                }
                elsif ($i == $inn && $games{$game}->{HRUNS} > $games{$game}->{ARUNS} && $games{$game}->{SCR1}->{$i} == 0) {
                    $value = 'X';
                }
                else {
                    $value = $games{$game}->{SCR1}->{$i};
                }
                print OUT "<td class=\"ic\" width=\"18\">$value</td>";
                $i++;
            }
            print OUT "<td class=\"icgb\" width=\"18\">$games{$game}->{HRUNS}</td>\n";
            print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{HHITS}</td>\n";
            print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{HERRS}</td></tr>\n";
            print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
            print OUT '<tr><td class="gl" colspan="17" style="padding:6px 4px 4px 4px;">';

            $j = $games{$game}->{WP};
            $value = $players{$j}->{FNAME};
            $value =~ s/^(.).*?$/$1/;
            print OUT "  W: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
	    if ($players{$j}->{TOTW}) {
		print OUT " ($players{$j}->{TOTW}-$players{$j}->{TOTL})";
	    }
            $j = $games{$game}->{LP};
            $value = $players{$j}->{FNAME};
            $value =~ s/^(.).*?$/$1/;
            print OUT "  L: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
	    if ($players{$j}->{TOTW}) {
		print OUT " ($players{$j}->{TOTW}-$players{$j}->{TOTL})";
	    }
            if ($games{$game}->{SP}) {
                $j = $games{$game}->{SP};
                $value = $players{$j}->{FNAME};
                $value =~ s/^(.).*?$/$1/;
                print OUT "  S: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
		if ($players{$j}->{TOTS}) {
		    print OUT " ($players{$j}->{TOTS})";
		}
            }
            print OUT '<br/>HR - ';
            $aHR = '';
            $hHR = '';
            foreach $player (keys %{$games{$game}->{HR}}) {
                $value = $players{$player}->{FNAME};
                $value =~ s/^(.).*?$/$1/;

                if ($players{$player}->{TEAM} == $games{$game}->{HTEAM}) {
                    $value = $players{$player}->{FNAME};
                    $value =~ s/^(.).*?$/$1/;
                    if ($hHR eq '') {
                        $hHR = "$teams{$players{$player}->{TEAM}}->{ABBREV}: ";
                    }
                    else {
                        $hHR .= ', ';
                    }
                    $hHR .= "<a href=\"$self->{INFO}->{URL}/players/player_$player.html\">$value. $players{$player}->{LNAME}</a>";
                    if ($games{$game}->{HR}->{$player} > 1) {
                        $hHR .= " $games{$game}->{HR}->{$player}";
                    }
		    if ($players{$player}->{TOTHR}) {
			$hHR .= " ($players{$player}->{TOTHR})";
		    }
                }
                else {
                    $value = $players{$player}->{FNAME};
                    $value =~ s/^(.).*?$/$1/;
                    if ($aHR eq '') {
                        $aHR = "$teams{$players{$player}->{TEAM}}->{ABBREV}: ";
                    }
                    else {
                        $aHR .= ', ';
                    }
                    $aHR .= "<a href=\"$self->{INFO}->{URL}/players/player_$player.html\">$value. $players{$player}->{LNAME}</a>";
                    if ($games{$game}->{HR}->{$player} > 1) {
                        $aHR .= " $games{$game}->{HR}->{$player}";
                    }
		    if ($players{$player}->{TOTHR}) {
			$aHR .= " ($players{$player}->{TOTHR})";
		    }
                }
            }
            if ($aHR eq '' && $hHR eq '') {
                print OUT 'NONE';
            }
            else {
                print OUT "$aHR $hHR";
            }
            print OUT '</td></tr></table></td></tr></table>';
            $gCount++;
        }

        print OUT '</td></tr></table>';
        print OUT '</td><td valign="top" width="100%"><table class="lpo" width="100%">';
        print OUT '<tr><td class="h1_l"> Upcoming Games </td></tr><tr><td colspan="2"><img src="'.$self->{INFO}->{URL}.'/images/pixel.gif" width="1" height="12"></td></tr>';

        $gCount = scalar @{$teams{$team}->{GAMES}} - $self->{SIMLENGTH};
        while ($gCount < scalar @{$teams{$team}->{GAMES}}) {
            $game = ${@{$teams{$team}->{GAMES}}}[$gCount];

	    if ($games{$game}->{MONTH} < $month || ($games{$game}->{MONTH} == $month && $games{$game}->{DAY} < $day)) {
		$gCount++;
                next;
            }

            $hTeam = $games{$game}->{HTEAM};
            $aTeam = $games{$game}->{ATEAM};
            print OUT '<tr><td align="left" valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:4px;">';
            print OUT OOTPOU::Utils::textDate("$games{$game}->{YEAR}-$games{$game}->{MONTH}-$games{$game}->{DAY}").'</td></tr>';

            print OUT '<tr><td align="left" valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:4px;"></td>';
#            print OUT '<td align="left" valign="top" style="padding-top:0px;padding-left:6px;padding-right:12px;padding-bottom:4px;"> </td></tr>';
            print OUT '<tr><td valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:12px;">';
            print OUT '<table cellspacing="0" cellpadding="0" width="423" class="lpo">';
            print OUT '<tr><td valign="top" width="44" style="padding-top:1px;padding-left:1px;">';
            print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$aTeam}->{LOGO}\" width=\"40\" height=\"40\"><br>";
            print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$hTeam}->{LOGO}\" width=\"40\" height=\"40\"></td>\n";
            print OUT '<td width="405" valign="top"><table cellspacing="0" cellpadding="1" width="380" style="border-left-width:1px; border-left-style:solid; border-left-color:#999999;">';
            print OUT '<tr><td class="hc" colspan="2">'.OOTPOU::Utils::convertTime($games{$game}->{TIME}).'</td></tr>';
            print OUT "<tr><td class=\"sl\" width=\"205\"><a href=\"$self->{INFO}->{URL}/teams/team_$aTeam.html\">$teams{$aTeam}->{CITY} $teams{$aTeam}->{NAME}</a></td><td class=\"sl\" width=\"200\">";
	    print OUT ($teams{$aTeam}->{HWINS} + $teams{$aTeam}->{AWINS}).'-'.($teams{$aTeam}->{HLOSS} + $teams{$aTeam}->{ALOSS});
	    print OUT ", On Road: $teams{$aTeam}->{AWINS}-$teams{$aTeam}->{ALOSS}</td></tr>";
            print OUT '<tr><td class="sl" width="205" style="border-top-width:1px; border-top-style:solid; border-top-color:#999999;">';
            print OUT "<a href=\"$self->{INFO}->{CSV}/teams/team_$hTeam.html\">$teams{$hTeam}->{CITY} $teams{$hTeam}->{NAME}</a></td>";
            print OUT '<td class="sl" width="200" style="border-top-width:1px; border-top-style:solid; border-top-color:#999999;">';
	    print OUT ($teams{$hTeam}->{HWINS} + $teams{$hTeam}->{AWINS}).'-'.($teams{$hTeam}->{HLOSS} + $teams{$hTeam}->{ALOSS});
	    print OUT ", At Home: $teams{$hTeam}->{HWINS}-$teams{$hTeam}->{HLOSS}</td></tr>\n";
            print OUT '</td></tr></table></td></tr></table>';
            $gCount++;
        }

        print OUT '</td></tr></table>';
        print OUT '</td></tr></table>';
        close (OUT);

	open (OUT, ">OOTPOU/lastSim/$leagueid/t".$team."stats.html");

	print OUT $teamChooser;
	print OUT '<tr><td><table width="100%" cellspacing="0" cellpadding="1" class="lpo" style="margin-bottom:12px;">';
	print OUT '<tr><td class="hsc2_l" style="width:180px;">Name</td><td class="hsn2">G</td><td class="hsn2">AB</td><td class="hsn2">R</td><td class="hsn2">H</td><td class="hsn2">2B</td><td class="hsn2">3B</td><td class="hsn2">HR</td><td class="hsn2">RBI</td><td class="hsn2">TB</td><td class="hsn2">BB</td><td class="hsn2">K</td><td class="hsn2">SB</td><td class="hsn2">CS</td><td class="hsn2">AVG</td><td class="hsn2">OBP</td><td class="hsn2">SLG</td><td class="hsn2">OPS</td></tr>';

	$c = 1;
	@keys = sort {$players{$a}->{LNAME} cmp $players{$b}->{LNAME}} keys %{$teams{$team}->{PLAYERS}};
	foreach $key (@keys) {
	    if ($players{$key}->{PA} > 0 || $players{$key}->{RBI} > 0 ||
		$players{$key}->{R} > 0 || $players{$key}->{SB} > 0 || $players{$key}->{CS} > 0) {
		$tb = $players{$key}->{H}+$players{$key}->{B2}+(2*$players{$key}->{B3})+(3*$players{$key}->{HR});
		if ($players{$key}->{AB} > 0) {
		    $avg = sprintf("%.3f", ($players{$key}->{H}/$players{$key}->{AB}));
		    $avg =~ s/^0\././;
		    $slg = sprintf("%.3f", ($tb/$players{$key}->{AB}));
		    $slg =~ s/^0\././;
		}
		else {
		    $avg = '.000';
		    $slg = '.000';
		}
		if (($players{$key}->{AB}+$players{$key}->{BB}+$players{$key}->{HBP}+$players{$key}->{SF}) > 0) {
		    $obp = sprintf("%.3f", (($players{$key}->{H}+$players{$key}->{BB}+$players{$key}->{HBP})/
					    ($players{$key}->{AB}+$players{$key}->{BB}+$players{$key}->{HBP}+$players{$key}->{SF})));
		    $obp =~ s/^0\././;
		}
		else {
		    $obp = '.000';
		}
		$ops = sprintf("%.3f", ($obp+$slg));
		$ops =~ s/^0\././;
		print OUT '<tr><td class="s'.$c."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$key.html\">$players{$key}->{FNAME} $players{$key}->{LNAME}</td>\n";
		print OUT "<td class=\"s$c\">$players{$key}->{G}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{AB}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{R}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{H}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{B2}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{B3}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{HR}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{RBI}</td>\n";
                print OUT "<td class=\"s$c\">$tb</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{BB}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{K}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{SB}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{CS}</td>\n";
                print OUT "<td class=\"s$c\">$avg</td>\n";
                print OUT "<td class=\"s$c\">$obp</td>\n";
                print OUT "<td class=\"s$c\">$slg</td>\n";
                print OUT "<td class=\"s$c\">$ops</td></tr>\n";
		if ($c == 1) {
		    $c = 2;
		}
		else {
		    $c = 1;
		}
		if ($players{$key}->{G} > $teams{$team}->{G}) {
		    $teams{$team}->{G} = $players{$key}->{G};
		}
		$teams{$team}->{AB}  += $players{$key}->{AB};
                $teams{$team}->{R}   += $players{$key}->{R};
                $teams{$team}->{H}   += $players{$key}->{H};
                $teams{$team}->{B2}  += $players{$key}->{B2};
                $teams{$team}->{B3}  += $players{$key}->{B3};
                $teams{$team}->{HR}  += $players{$key}->{HR};
                $teams{$team}->{RBI} += $players{$key}->{RBI};
                $teams{$team}->{BB}  += $players{$key}->{BB};
                $teams{$team}->{K}   += $players{$key}->{K};
                $teams{$team}->{SB}  += $players{$key}->{SB};
                $teams{$team}->{CS}  += $players{$key}->{CS};
                $teams{$team}->{HBP} += $players{$key}->{HBP};
                $teams{$team}->{SF}  += $players{$key}->{SF};

		$players{$key}->{SCORE} = $players{$key}->{H} + ($players{$key}->{HR}*4) + $players{$key}->{RBI} + $players{$key}->{R} + $players{$key}->{SB};
	    }
	}

	$tb = $teams{$team}->{H}+$teams{$team}->{B2}+(2*$teams{$team}->{B3})+(3*$teams{$team}->{HR});
	if ($teams{$team}->{AB} > 0) {
	    $avg = sprintf("%.3f", ($teams{$team}->{H}/$teams{$team}->{AB}));
	    $avg =~ s/^0\././;
	    $slg = sprintf("%.3f", ($tb/$teams{$team}->{AB}));
	    $slg =~ s/^0\././;
	}
	else {
	    $avg = '.000';
	    $slg = '.000';
	}
	if ($teams{$team}->{AB}+$teams{$team}->{BB}+$teams{$team}->{HBP}+$teams{$team}->{SF} > 0) {
	    $obp = sprintf("%.3f", (($teams{$team}->{H}+$teams{$team}->{BB}+$teams{$team}->{HBP})/
				    ($teams{$team}->{AB}+$teams{$team}->{BB}+$teams{$team}->{HBP}+$teams{$team}->{SF})));
	    $obp =~ s/^0\././;
	}
	else {
	    $obp = '.000';
	}
	$ops = sprintf("%.3f", ($obp+$slg));
	$ops =~ s/^0\././;
	print OUT '<tr><td class="hsc2_l">Total</td>';
	print OUT "<td class=\"hsn2\">$teams{$team}->{G}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{AB}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{R}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{H}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{B2}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{B3}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{HR}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{RBI}</td>\n";
	print OUT "<td class=\"hsn2\">$tb</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{BB}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{K}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{SB}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{CS}</td>\n";
	print OUT "<td class=\"hsn2\">$avg</td>\n";
	print OUT "<td class=\"hsn2\">$obp</td>\n";
	print OUT "<td class=\"hsn2\">$slg</td>\n";
	print OUT "<td class=\"hsn2\">$ops</td></tr>\n";
	print OUT '</table>';

	print OUT '<tr><td>&nbsp;</td></tr>';
	print OUT '<tr><td><table width="100%" cellspacing="0" cellpadding="1" class="lpo" style="margin-bottom:12px;"></td></tr>';
	print OUT '<tr><td class="hsc2_l" style="width:180px;">Name</td><td class="hsn2">W</td><td class="hsn2">L</td><td class="hsn2">SV</td><td class="hsn2">CG</td><td class="hsn2">SO</td><td class="hsn2">ERA</td><td class="hsn2">G</td><td class="hsn2">GS</td><td class="hsn2">IP</td><td class="hsn2">HA</td><td class="hsn2">R</td><td class="hsn2">ER</td><td class="hsn2">HR</td><td class="hsn2">BB</td><td class="hsn2">K</td><td class="hsn2">WHIP</td><td class="hsn2">OAVG</td><td class="hsn2">BABIP</td></tr>';
	$c = 1;
	foreach $key (@keys) {
	    if ($players{$key}->{BF} > 0) {
		print OUT '<tr><td class="s'.$c."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$key.html\">$players{$key}->{FNAME} $players{$key}->{LNAME}</td>\n";

		$players{$key}->{IP} += int($players{$key}->{IPF}/3);
		$players{$key}->{IPF} = $players{$key}->{IPF}%3;

		if ($players{$key}->{PAB} > 0) {
		    $avg =  sprintf("%.3f", ($players{$key}->{PH}/$players{$key}->{PAB}));
		    $avg =~ s/^0\././;
		}
		else {
		    $avg = '.000';
		}
		if ($players{$key}->{IP} > 0 || $players{$key}->{IPF} > 0) {
		    $obp = ($players{$key}->{PH} + $players{$key}->{PBB})/($players{$key}->{IP} + ($players{$key}->{IPF}/3));
		    $obp = sprintf("%.2f", $obp);
		}
		elsif ($players{$key}->{PH} + $players{$key}->{PBB} == 0) {
		    $obp = '0.00';
		}
		else {
		    $obp = 'Inf.';
		}
		if ($players{$key}->{IP} > 0 || $players{$key}->{IPF} > 0) {
		    $slg = 9*($players{$key}->{PER})/($players{$key}->{IP} + ($players{$key}->{IPF}/3));
		    $slg = sprintf("%.2f", $slg);
		}
                elsif ($players{$key}->{PER} == 0) {
                    $slg = '0.00';
                }
                else {
                    $slg = 'Inf.';
                }
		if ($players{$key}->{PAB} - $players{$key}->{PHR} - $players{$key}->{PK} + $players{$key}->{PSF} > 0) {
		    $ops = sprintf("%.3f", ($players{$key}->{PH} - $players{$key}->{PHR})/($players{$key}->{PAB} - $players{$key}->{PHR} - $players{$key}->{PK} + $players{$key}->{PSF}));
		    $ops =~ s/^0\././;
		}
		else {
		    $ops = '0.000';
		}
		print OUT "<td class=\"s$c\">$players{$key}->{W}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{L}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{S}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{CG}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{SO}</td>\n";
                print OUT "<td class=\"s$c\">$slg</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PG}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PGS}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{IP}.$players{$key}->{IPF}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PH}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PR}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PER}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PHR}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PBB}</td>\n";
                print OUT "<td class=\"s$c\">$players{$key}->{PK}</td>\n";
                print OUT "<td class=\"s$c\">$obp</td>\n";
                print OUT "<td class=\"s$c\">$avg</td>\n";
                print OUT "<td class=\"s$c\">$ops</td></tr>\n";

		if ($players{$key}->{IP} >= 7 || ($players{$key}->{S} >= 2 && $players{$key}->{IP} > 2)) {
		    $players{$key}->{PSCORE} = 2*$players{$key}->{IP} + (4*$players{$key}->{W}) - (4*$players{$key}->{L}) + (4*$players{$key}->{S})
			+ 4*(5-$slg) + 2*$players{$key}->{SO} - $players{$key}->{PH} - $players{$key}->{PBB};
		}

		$teams{$team}->{IP}  += $players{$key}->{IP};
		$teams{$team}->{IPF} += $players{$key}->{IPF};
		$teams{$team}->{PGS} += $players{$key}->{PGS};
		$teams{$team}->{PAB} += $players{$key}->{PAB};
		$teams{$team}->{PH}  += $players{$key}->{PH};
		$teams{$team}->{PBB} += $players{$key}->{PBB};
		$teams{$team}->{PK}  += $players{$key}->{PK};
		$teams{$team}->{PR}  += $players{$key}->{PR};
		$teams{$team}->{PER} += $players{$key}->{PER};
		$teams{$team}->{W}   += $players{$key}->{W};
		$teams{$team}->{L}   += $players{$key}->{L};
		$teams{$team}->{S}   += $players{$key}->{S};
		$teams{$team}->{CG}  += $players{$key}->{CG};
		$teams{$team}->{SO}  += $players{$key}->{SO};
		$teams{$team}->{PHR} += $players{$key}->{PHR};
		$teams{$team}->{PSF} += $players{$key}->{PSF};

		if ($c == 1) {
                    $c = 2;
                }
                else {
                    $c = 1;
                }
	    }

	}
	
	$teams{$team}->{IP} += int($teams{$team}->{IPF}/3);
	$teams{$team}->{IPF} = $teams{$team}->{IPF}%3;

	if ($teams{$team}->{PAB} > 0) {
	    $avg =  sprintf("%.3f", ($teams{$team}->{PH}/$teams{$team}->{PAB}));
	    $avg =~ s/^0\././;
	}
	else {
	    $avg = '.000';
	}
	if ($teams{$team}->{IP} > 0 || $teams{$team}->{IPF} > 0) {
	    $obp = ($teams{$team}->{PH} + $teams{$team}->{PBB})/($teams{$team}->{IP} + ($teams{$team}->{IPF}/3));
	    $obp = sprintf("%.2f", $obp);
	}
	elsif ($teams{$team}->{PH} + $teams{$team}->{PBB} == 0) {
	    $obp = '0.00';
	}
	else {
	    $obp = 'Inf.';
	}
	if ($teams{$team}->{IP} > 0 || $teams{$team}->{IPF} > 0) {
	    $slg = 9*($teams{$team}->{PER})/($teams{$team}->{IP} + ($teams{$team}->{IPF}/3));
	    $slg = sprintf("%.2f", $slg);
	}
	elsif ($teams{$team}->{PER} == 0) {
	    $slg = '0.00';
	}
	else {
	    $slg = 'Inf.';
	}
	if ($teams{$team}->{IP} > 0 || $teams{$team}->{IPF} > 0) {
	    $slg = 9*($teams{$team}->{PER})/($teams{$team}->{IP} + ($teams{$team}->{IPF}/3));
	    $slg = sprintf("%.2f", $slg);
	}
	elsif ($teams{$team}->{PER} == 0) {
	    $slg = '0.00';
	}
	else {
	    $slg = 'Inf.';
	}
	if ($teams{$team}->{PAB} - $teams{$team}->{PHR} - $teams{$team}->{PK} + $teams{$team}->{PSF} > 0) {
	    $ops = sprintf("%.3f", ($teams{$team}->{PH} - $teams{$team}->{PHR})/($teams{$team}->{PAB} - $teams{$team}->{PHR} - $teams{$team}->{PK} + $teams{$team}->{PSF}));
	    $ops =~ s/^0\././;
	}
	else {
	    $ops = '0.000';
	}
	print OUT '<tr><td class="hsc2_l">Total</td>';
	print OUT "<td class=\"hsn2\">$teams{$team}->{W}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{L}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{S}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{CG}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{SO}</td>\n";
	print OUT "<td class=\"hsn2\">$slg</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{G}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PGS}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{IP}.$teams{$team}->{IPF}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PH}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PR}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PER}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PHR}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PBB}</td>\n";
	print OUT "<td class=\"hsn2\">$teams{$team}->{PK}</td>\n";
	print OUT "<td class=\"hsn2\">$obp</td>\n";
	print OUT "<td class=\"hsn2\">$avg</td>\n";
	print OUT "<td class=\"hsn2\">$ops</td></tr>\n";

	print OUT '</table></tr></td>';
	close (OUT);
    }

    open (OUT, ">OOTPOU/lastSim/$leagueid/perfs.html");
    print OUT '<tr><td><table width="100%">';
    print OUT '<tr><td valign="top"><table class="lpo" width="460">';
    print OUT '<tr><td class="h1_l"> Last Sim\'s Top Games </td></tr><tr><td colspan="2"><img src="'.$self->{INFO}->{URL}.'/images/pixel.gif" width="1" height="12"></td></tr>';

    $gCount = 1;
    @keys = sort {$games{$b}->{SCORE} <=> $games{$a}->{SCORE}} keys %games;
    my @perfKeys;
    foreach $game (@keys) {
	push @perfKeys, $game;
        $gCount++;
	if ($gCount > 7) {
	    last;
	}
    }
    @perfKeys = sort {(($games{$a}->{MONTH}*100) + $games{$a}->{DAY}) <=> (($games{$b}->{MONTH}*100) + $games{$b}->{DAY})} @perfKeys;
    
    foreach $game (@perfKeys) {
	if (!$game) {
	    last;
	}
	$hTeam = $games{$game}->{HTEAM};
	$aTeam = $games{$game}->{ATEAM};

	print OUT '<tr><td align="left" valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:4px;">';
	print OUT OOTPOU::Utils::textDate("$games{$game}->{YEAR}-$games{$game}->{MONTH}-$games{$game}->{DAY}").': ';
	print OUT "<a href=\"$self->{INFO}->{URL}/box_scores/game_box_$game.html\">Box Scores</a> | ";
	print OUT "<a href=\"$self->{INFO}->{URL}/game_logs/log_$game.html\">Game Log</a> </td>\n";
#	print OUT '<td align="left" valign="top" style="padding-top:0px;padding-left:6px;padding-right:12px;padding-bottom:4px;"> </td></tr>';
	print OUT '<tr><td valign="top" style="padding-top:0px;padding-left:12px;padding-right:6px;padding-bottom:12px;">';
	print OUT '<table cellspacing="0" cellpadding="0" width="423" class="lpo">';
	print OUT '<tr><td valign="top" width="44" style="padding-top:1px;padding-left:1px;">';
	print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$aTeam}->{LOGO}\" width=\"40\" height=\"40\">";
	print OUT "<img src=\"$self->{INFO}->{URL}/images/$teams{$hTeam}->{LOGO}\" width=\"40\" height=\"40\"></td>\n";
	print OUT '<td width="405"><table cellspacing="0" cellpadding="1" width="380" style="border-left-width:1px; border-left-style:solid; border-left-color:#999999;">';
#	print OUT "<tr><td class=\"hc\">$games{$game}->{SCORE}</td>\n";
	print OUT '<tr><td class="hc">&#160;</td>';

	$inn = $games{$game}->{INNINGS};
	$totInn = $inn;
	if ($totInn > 10) {
	    $i = $totInn-10;
	}
	else {
	    $totInn = 10;
	    $i = 1;
	}
	$j = $i;
	while ($i <= $totInn) {
	    print OUT "<td class=\"hc\" width=\"18\">$i</td>\n";
	    $i++;
	}
	print OUT '<td class="hc" width="18">R</td><td class="hc" width="18">H</td><td class="hc" width="18">E</td></tr>';
	print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
	print OUT "<a href=\"$self->{INFO}->{URL}/teams/team_$aTeam.html\">$teams{$aTeam}->{CITY}</a></td>\n";
	$i = $j;
	while ($i <= $totInn) {
	    if ($i > $inn) {
		$value = '&#160;';
	    }
	    else {
		$value = $games{$game}->{SCR0}->{$i};
	    }
	    print OUT "<td class=\"ic\" width=\"18\">$value</td>";
	    $i++;
	}
	print OUT "<td class=\"icgb\" width=\"18\">$games{$game}->{ARUNS}</td>\n";
	print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{AHITS}</td>\n";
	print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{AERRS}</td></tr>\n";
	print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
	print OUT "<a href=\"$self->{INFO}->{URL}/teams/team_$hTeam.html\">$teams{$hTeam}->{CITY}</a></td>\n";
	$i = $j;
	while ($i <= $totInn) {
	    if ($i > $inn) {
		$value = '&#160;';
	    }
	    elsif ($i == $inn && $games{$game}->{HRUNS} > $games{$game}->{ARUNS} && $games{$game}->{SCR1}->{$i} == 0) {
		$value = 'X';
	    }
	    else {
		$value = $games{$game}->{SCR1}->{$i};
	    }
	    print OUT "<td class=\"ic\" width=\"18\">$value</td>";
	    $i++;
	}
	print OUT "<td class=\"icgb\" width=\"18\">$games{$game}->{HRUNS}</td>\n";
	print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{HHITS}</td>\n";
	print OUT "<td class=\"ic\" width=\"18\">$games{$game}->{HERRS}</td></tr>\n";
	print OUT '<tr><td class="sl" style="border-bottom-width:1px; border-bottom-style:solid; border-bottom-color:#999999;">';
	print OUT '<tr><td class="gl" colspan="17" style="padding:6px 4px 4px 4px;">';
	
	$j = $games{$game}->{WP};
	$value = $players{$j}->{FNAME};
	$value =~ s/^(.).*?$/$1/;
	print OUT "  W: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
	if ($players{$j}->{TOTW}) {
	    print OUT " ($players{$j}->{TOTW}-$players{$j}->{TOTL})";
	}
	$j = $games{$game}->{LP};
	$value = $players{$j}->{FNAME};
	$value =~ s/^(.).*?$/$1/;
	print OUT "  L: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
	if ($players{$j}->{TOTW}) {
	    print OUT " ($players{$j}->{TOTW}-$players{$j}->{TOTL})";
	}
	if ($games{$game}->{SP}) {
	    $j = $games{$game}->{SP};
	    $value = $players{$j}->{FNAME};
	    $value =~ s/^(.).*?$/$1/;
	    print OUT "  S: <a href=\"$self->{INFO}->{URL}/players/player_$j.html\">$value. $players{$j}->{LNAME}</a>";
	    if ($players{$j}->{TOTS}) {
		print OUT " ($players{$j}->{TOTS})";
	    }
	}
	print OUT '<br/>HR - ';
	$aHR = '';
	$hHR = '';
	foreach $player (keys %{$games{$game}->{HR}}) {
	    $value = $players{$player}->{FNAME};
	    $value =~ s/^(.).*?$/$1/;
	    
	    if ($players{$player}->{TEAM} == $games{$game}->{HTEAM}) {
		$value = $players{$player}->{FNAME};
		$value =~ s/^(.).*?$/$1/;
		if ($hHR eq '') {
		    $hHR = "$teams{$players{$player}->{TEAM}}->{ABBREV}: ";
		}
		else {
		    $hHR .= ', ';
		}
		$hHR .= "<a href=\"$self->{INFO}->{URL}/players/player_$player.html\">$value. $players{$player}->{LNAME}</a>";
		if ($games{$game}->{HR}->{$player} > 1) {
		    $hHR .= " $games{$game}->{HR}->{$player}";
		}
		if ($players{$player}->{TOTHR}) {
		    $hHR .= " ($players{$player}->{TOTHR})";
		}
	    }
	    else {
		$value = $players{$player}->{FNAME};
		$value =~ s/^(.).*?$/$1/;
		if ($aHR eq '') {
		    $aHR = "$teams{$players{$player}->{TEAM}}->{ABBREV}: ";
		}
		else {
		    $aHR .= ', ';
		}
		$aHR .= "<a href=\"$self->{INFO}->{URL}/players/player_$player.html\">$value. $players{$player}->{LNAME}</a>";
		if ($games{$game}->{HR}->{$player} > 1) {
		    $aHR .= " $games{$game}->{HR}->{$player}";
		}
		if ($players{$player}->{TOTHR}) {
		    $aHR .= " ($players{$player}->{TOTHR})";
		}
	    }
	}
	if ($aHR eq '' && $hHR eq '') {
	    print OUT 'NONE';
	}
	else {
	    print OUT "$aHR $hHR";
	}
	print OUT '</td></tr></table></td></tr></table>';
    }

    print OUT '</td></tr></table>';
    print OUT '</td><td valign="top"><table width="435" class="lpo">';
    print OUT '<tr><td class="h1_l" width="100%"> Last Sim\'s Top Performers </td></tr><tr><td><img src="'.$self->{INFO}->{URL}.'/images/pixel.gif" width="1" height="12"></td></tr>';

    print OUT '<tr><td><table width="100%" cellspacing="0" cellpadding="1" class="lpo" style="margin-bottom:12px;">';
    print OUT '<tr><td class="hsc2_l" style="width:180px;">Name</td><td class="hsn2">AVG</td><td class="hsn2">R</td><td class="hsn2">H</td><td class="hsn2">HR</td><td class="hsn2">RBI</td><td class="hsn2">SB</td><td class="hsn2">OPS</td></tr>';

    $c = 1;
    @perfKeys = [];
    @keys = sort {$players{$b}->{SCORE} <=> $players{$a}->{SCORE}} keys %players;
    foreach $key (@keys) {
	push @perfKeys, $key;
	$c++;
	if ($c > 20) {
	    last;
	}
    }

    $c = 1;
    @perfKeys = sort {$players{$a}->{LNAME} cmp $players{$b}->{LNAME}} @perfKeys;
    foreach $key (@perfKeys) {
	if ($players{$key}->{PA} > 0 || $players{$key}->{RBI} > 0 ||
	    $players{$key}->{R} > 0 || $players{$key}->{SB} > 0 || $players{$key}->{CS} > 0) {
	    $tb = $players{$key}->{H}+$players{$key}->{B2}+(2*$players{$key}->{B3})+(3*$players{$key}->{HR});
	    if ($players{$key}->{AB} > 0) {
		$avg = sprintf("%.3f", ($players{$key}->{H}/$players{$key}->{AB}));
		$avg =~ s/^0\././;
		$slg = sprintf("%.3f", ($tb/$players{$key}->{AB}));
		$slg =~ s/^0\././;
	    }
	    else {
		$avg = '.000';
		$slg = '.000';
	    }
	    if (($players{$key}->{AB}+$players{$key}->{BB}+$players{$key}->{HBP}+$players{$key}->{SF}) > 0) {
		$obp = sprintf("%.3f", (($players{$key}->{H}+$players{$key}->{BB}+$players{$key}->{HBP})/
					($players{$key}->{AB}+$players{$key}->{BB}+$players{$key}->{HBP}+$players{$key}->{SF})));
		$obp =~ s/^0\././;
	    }
	    else {
		$obp = '.000';
	    }
	    $ops = sprintf("%.3f", ($obp+$slg));
	    $ops =~ s/^0\././;
	    print OUT '<tr><td class="s'.$c."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$key.html\">$players{$key}->{FNAME} $players{$key}->{LNAME}</a>, $teams{$players{$key}->{TEAM}}->{ABBREV}</td>\n";
            print OUT "<td class=\"s$c\">$avg</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{R}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{H}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{HR}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{RBI}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{SB}</td>\n";
	    print OUT "<td class=\"s$c\">$ops</td></tr>\n";
	    if ($c == 1) {
		$c = 2;
	    }
	    else {
		$c = 1;
	    }
	}       
    }
    print OUT '</td></tr></table>';


    print OUT '<tr><td>&nbsp;</td></tr>';
    print OUT '<tr><td><table width="100%" cellspacing="0" cellpadding="1" class="lpo" style="margin-bottom:12px;"></td></tr>';
        print OUT '<tr><td class="hsc2_l" style="width:180px;">Name</td><td class="hsn2">W</td><td class="hsn2">L</td><td class="hsn2">SV</td><td class="hsn2">IP</td><td class="hsn2">ERA</td><td class="hsn2">K</td><td class="hsn2">WHIP</td><td class="hsn2">OAVG</td></tr>';

    $c = 1;
    @perfKeys = [];
    @keys = sort {$players{$b}->{PSCORE} <=> $players{$a}->{PSCORE}} keys %players;
    foreach $key (@keys) {
        push @perfKeys, $key;
        $c++;
        if ($c > 20) {
            last;
        }
    }

    $c = 1;
    @perfKeys = sort {$players{$a}->{LNAME} cmp $players{$b}->{LNAME}} @perfKeys;

    foreach $key (@perfKeys) {
	if ($players{$key}->{BF} > 0) {
	    print OUT '<tr><td class="s'.$c."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$key.html\">$players{$key}->{FNAME} $players{$key}->{LNAME}</a>";
	    print OUT ", $teams{$players{$key}->{TEAM}}->{ABBREV}</td>\n";

	    $players{$key}->{IP} += int($players{$key}->{IPF}/3);
	    $players{$key}->{IPF} = $players{$key}->{IPF}%3;

	    if ($players{$key}->{PAB} > 0) {
		$avg =  sprintf("%.3f", ($players{$key}->{PH}/$players{$key}->{PAB}));
		$avg =~ s/^0\././;
	    }
	    else {
		$avg = '.000';
	    }
	    
	    if ($players{$key}->{IP} > 0 || $players{$key}->{IPF} > 0) {
		$obp = ($players{$key}->{PH} + $players{$key}->{PBB})/($players{$key}->{IP} + ($players{$key}->{IPF}/3));
		$obp = sprintf("%.2f", $obp);
	    }
	    elsif ($players{$key}->{PH} + $players{$key}->{PBB} == 0) {
		$obp = '0.00';
	    }
	    else {
		$obp = 'Inf.';
	    }
	    if ($players{$key}->{IP} > 0 || $players{$key}->{IPF} > 0) {
		$slg = 9*($players{$key}->{PER})/($players{$key}->{IP} + ($players{$key}->{IPF}/3));
		$slg = sprintf("%.2f", $slg);
	    }
	    elsif ($players{$key}->{PER} == 0) {
		$slg = '0.00';
	    }
	    else {
		$slg = 'Inf.';
	    }
	    
	    print OUT "<td class=\"s$c\">$players{$key}->{W}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{L}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{S}</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{IP}.$players{$key}->{IPF}</td>\n";
	    print OUT "<td class=\"s$c\">$slg</td>\n";
	    print OUT "<td class=\"s$c\">$players{$key}->{PK}</td>\n";
	    print OUT "<td class=\"s$c\">$obp</td>\n";
	    print OUT "<td class=\"s$c\">$avg</td></tr>\n";
	    
            if ($c == 1) {
                $c = 2;
            }
            else {
                $c = 1;
            }
	}
    }

    print OUT '</td></tr></table>';
    print OUT '</td></tr></table>';
    close (OUT);

    open (OUT, '>OOTPOU/lastSim/info.txt');
    print OUT "SIMLENGTH:$self->{SIMLENGTH}\n";
    print OUT "LS_DATE:$year-$month-$day\n";
    print OUT 'LS_TIME:'.(localtime)."\n";
    print OUT "PSFORMAT:$self->{PSFORMAT}\n";
    print OUT "WINPCT:$self->{WINPCT}\n";
    print OUT "PYTHEXP:$self->{PYTHEXP}\n";
    print OUT "SEASONS:$self->{SEASONS}\n";
    close (OUT);

    $self->{LS_TIME} = localtime;
    $self->{LS_DATE} = "$year-$month-$day";
}

sub saveInfo {
    my ($self) = @_;
    open (OUT, '>OOTPOU/lastSim/info.txt');
    print OUT "SIMLENGTH:$self->{PARAMS}->{SIMLENGTH}\n";
    print OUT "LS_DATE:$self->{LS_DATE}\n";
    print OUT "LS_TIME:$self->{LS_TIME}\n";
    print OUT "PSFORMAT:$self->{PARAMS}->{PSFORMAT}\n";
    print OUT "WINPCT:$self->{PARAMS}->{WINPCT}\n";
    print OUT "PYTHEXP:$self->{PARAMS}->{PYTHEXP}\n";
    print OUT "SEASONS:$self->{PARAMS}->{SEASONS}\n";
    close (OUT);

    $self->{SIMLENGTH} = $self->{PARAMS}->{SIMLENGTH};
    $self->{PSFORMAT}  = $self->{PARAMS}->{PSFORMAT};
    $self->{WINPCT}    = $self->{PARAMS}->{WINPCT};
    $self->{PYTHEXP}   = $self->{PARAMS}->{PYTHEXP};
    $self->{SEASONS}   = $self->{PARAMS}->{SEASONS};
}

sub getInfo {
    my ($self) = @_;
    if (open (IN, "OOTPOU/lastSim/info.txt")) {
        while (<IN>) {
            chomp;
            if (/(\w+):(.*?)$/) {
                $self->{$1} = $2;
            }
        }
        close(IN);
    }
    if (!$self->{LS_TIME}) {
        $self->{LS_TIME} = 'N/A';
        $self->{LS_DATE} = 'N/A';
    }
    close (IN);
}

sub getDateRange {
    my ($self) = @_;

    my $minDate = $self->{INFO}->{DATE}->{DAY} - $self->{SIMLENGTH};
    my $minMonth = $self->{INFO}->{DATE}->{MONTH};
    if ($minDate <= 0) {
        $minMonth--;
        $minDate = 31+$minDate;
        if (469 =~ /$minMonth/) {
            $minDate--;
        }
    }
    $minDate = ($minMonth*100)+$minDate;

    my $maxDate = $self->{INFO}->{DATE}->{DAY} + $self->{SIMLENGTH} - 1;
    my $maxMonth = $self->{INFO}->{DATE}->{MONTH};
    my $dayMonth = 31;
    if (469 =~ /$maxMonth/) {
        $dayMonth = 30;
    }
    if ($maxDate > $dayMonth) {
        $maxMonth++;
        $maxDate = $maxDate - $dayMonth;
    }
    $maxDate = ($maxMonth*100)+$maxDate;

    return ($minDate, $maxDate);
}


sub simSeasons {
    my ($self, $leagueid) = @_;

    my %teams;
    my %subLeagues;
    my %divisions;
    my $psFormat = $self->{PSFORMAT};
    my $usePyth  = $self->{WINPCT};
    my $pythExp  = $self->{PYTHEXP};
    my $seasons  = $self->{SEASONS};
    my $methodText;
    my $team;
    my $cnt;
    my $nDivs;

    if ($usePyth == 1) {
	$methodText = "pythagorean record (exp=$pythExp)";
    }
    elsif ($usePyth == 2) {
    $methodText = "pythagorean record (exp=((r + ra)/g)^0.287)";
    }
    else {
	$methodText = 'actual record';
    }

    open (OUT, ">OOTPOU/lastSim/$leagueid/PlayoffOdds.html");
    print OUT "<tr><td class=\"h1\_l\" colspan=\"8\">Simulated remainder of $seasons seasons using $methodText to estimate success.</td></tr>";

    ###########  Load Teams  ###########
    open(IN, "$self->{INFO}->{CSV}/teams.csv") or die("Cannot open $self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/,0,"[^,]*?"\s*$/ && /^(\d+),"(.*?)","\w+","(.*?)",".*?","(.*?)",\d+,\d+,$leagueid,(\d+),/) {
            $teams{$1} = {};
            $teams{$1}->{CITY}      = $2;
            $teams{$1}->{NAME}      = $3;
            $teams{$1}->{IMAGE}     = $4;
            $teams{$1}->{SUBLEAGUE} = $5;

	    $teams{$1}->{WINS} = 0;
	    $teams{$1}->{LOSSES} = 0;
	    $teams{$1}->{RUNSFOR} = 0;
	    $teams{$1}->{RUNSAGAINST} = 0;
        }
    }
    close(IN);

    ###########  Load Current Records  ###########
    open(IN, "$self->{INFO}->{CSV}/games.csv") or die("Cannot open $self->{INFO}->{CSV}/games.csv");
    while (<IN>) {
#            1 ,   2                     , 3   , 4   , 5 , 6           ,7  ,8,9, 10, 11, 12  , 13  ,
#                                           1  ,  2  ,                                    3  ,  4,
#      if (/^\d+,$leagueid,(\d+),(\d+),\d+,"\d+-\d+-\d+",\d+,\d+,\d+,\d+,\d+,(\d+),(\d+),/) {
        if (/^\d+,$leagueid,(\d+),(\d+),\d+,".*?",\d+,0,1,\d+,\d+,(\d+),(\d+),/) {
	    $teams{$1}->{RUNSFOR}     += $4;
	    $teams{$1}->{RUNSAGAINST} += $3;
            $teams{$2}->{RUNSFOR}     += $3;
	    $teams{$2}->{RUNSAGAINST} += $4;
	    if ($4 > $3) {
		$teams{$1}->{WINS} += 1;
		$teams{$2}->{LOSSES} += 1;
	    }
	    else {
		$teams{$2}->{WINS} += 1;
		$teams{$1}->{LOSSES} += 1;
	    }

#           print OUT "Game $2 ($4) at $1 ($3) - Team $2 is $teams{$2}->{WINS}\-$teams{$2}->{LOSSES} - Team $1 is $teams{$1}->{WINS}\-$teams{$1}->{LOSSES}";
        }
    }
    close (IN);


    ###########  Set Team Pythags  ###########
    open(IN, "$self->{INFO}->{CSV}/teams.csv") or die("Cannot open $self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/,0,"[^,]*?"\s*$/ && /^(\d+),"(.*?)","\w+","(.*?)",".*?","(.*?)",\d+,\d+,$self->{INFO}->{LEAGUEID},(\d+),/) {
	    $teams{$1}->{WINPCT}    = OOTPOU::Utils::winPct($teams{$1}->{WINS},$teams{$1}->{LOSSES});
	    if ($usePyth == 1) {
        $teams{$1}->{PYTHPCT}   = OOTPOU::Utils::PythWinPct($teams{$1}->{RUNSFOR},$teams{$1}->{RUNSAGAINST},$pythExp);
	    }
	    else {
	       $teams{$1}->{PYTHPCT}   = OOTPOU::Utils::PythpatWinPct($teams{$1}->{RUNSFOR},$teams{$1}->{RUNSAGAINST},$teams{$1}->{WINS},$teams{$1}->{LOSSES});
      }
	    ($teams{$1}->{PWINS}, $teams{$1}->{PLOSSES}, $teams{$1}->{PDIFF}) = OOTPOU::Utils::pythRec($teams{$1}->{WINS},$teams{$1}->{LOSSES},$teams{$1}->{PYTHPCT});
	    $teams{$1}->{MINWINS} = 9999;
        }
    }
    close(IN);

    ###########  Load Subleague Info  ###########
    open(IN, "$self->{INFO}->{CSV}/sub_leagues.csv") or die("Cannot open $self->{INFO}->{CSV}/sub_leagues.csv");
    while (<IN>) {
        if (/^$leagueid,(\d+),"(.*?)",/) {
            $subLeagues{$1} = {};
            $subLeagues{$1}->{NAME} = $2;
            $subLeagues{$1}->{nDIVS} = 0;
        }
    }
    close(IN);
    ###########  Load Division Info  ###########
    open(IN, "$self->{INFO}->{CSV}/divisions.csv") or die("Cannot open $self->{INFO}->{CSV}/divisions.csv");
    while (<IN>) {
        if (/^$leagueid,(\d+),(\d+),"(.*?)",/) {
            $divisions{"$1-$2"} = {};
            $divisions{"$1-$2"}->{NAME} = $3;
            $subLeagues{$1}->{nDIVS} = $subLeagues{$1}->{nDIVS}+1; 
	    $nDivs++;
        }
    }
    close (IN);

    ###########  Load League Structure  ###########
    open(IN, "$self->{INFO}->{CSV}/team_relations.csv") or die("Cannot open $self->{INFO}->{CSV}/team_relations.csv");
    while (<IN>) {
        if (/^$leagueid,(\d+),(\d+),(\d+)\s*$/) {
            $teams{$3}->{DIVISION} = "$1-$2";
        }
    }
    close (IN);

    ###########  Simulate Seasons  ###########
    my $ind;
    my $randWin;
    my $homeWinPct;
    my $division;
    my $subleague;
    my $nWCs;

    for ($ind=1; $ind<($seasons+1); ++$ind) {
	foreach $team (keys %teams) {
	    $teams{$team}->{SIMWINS} = 0;
	    $teams{$team}->{SIMLOSSES} = 0;
        }

        open(IN, "$self->{INFO}->{CSV}/games.csv") or die("Cannot open $self->{INFO}->{CSV}/games.csv");
        while (<IN>) {
##                1 ,   2                     , 3   , 4   , 5 , 6   ,7  ,8,9,
##                                               1  ,  2  ,
	    if (/^\d+,$leagueid,(\d+),(\d+),\d+,"\d+-\d+-\d+",\d+,0,0,\d+,\d+,0,0,/) {
		$randWin = rand();

		if ($usePyth == 1 || ($usePyth == 2)) {
		    $homeWinPct = OOTPOU::Utils::log5WinPct($teams{$1}->{PYTHPCT}, $teams{$2}->{PYTHPCT});
		}
		else {
		    $homeWinPct = OOTPOU::Utils::log5WinPct($teams{$1}->{WINPCT}, $teams{$2}->{WINPCT});
		}

		if ($randWin < $homeWinPct) {
		    $teams{$1}->{SIMWINS} += 1;
		    $teams{$2}->{SIMLOSSES} += 1;
		}
		else {
		    $teams{$2}->{SIMWINS} += 1;
		    $teams{$1}->{SIMLOSSES} += 1;
		}
	    }
        }
        close (IN);

	foreach $team (keys %teams) {
	    $teams{$team}->{DIVWINNER} = 0;
	    $teams{$team}->{WC1WINNER} = 0;
	    $teams{$team}->{WC2WINNER} = 0;

	    $teams{$team}->{SEASONWINS} = $teams{$team}->{WINS} + $teams{$team}->{SIMWINS};
	    $teams{$team}->{SEASONLOSSES} = $teams{$team}->{LOSSES} + $teams{$team}->{SIMLOSSES};

	    $teams{$team}->{TOTALWINS} += $teams{$team}->{SEASONWINS};
	    $teams{$team}->{TOTALLOSSES} += $teams{$team}->{SEASONLOSSES};

	    if ($teams{$team}->{SEASONWINS} > $teams{$team}->{MAXWINS}) {
		$teams{$team}->{MAXWINS} = $teams{$team}->{SEASONWINS};
	    }
	    if ($teams{$team}->{SEASONWINS} < $teams{$team}->{MINWINS}) {
		$teams{$team}->{MINWINS} = $teams{$team}->{SEASONWINS};
	    }

        }  #End of foreach teams loop to finalize season stats

       ###########  Find Max Wins in Division  ###########
	foreach $division (sort {$a cmp $b} (keys %divisions)) {
	    $divisions{$division}->{DIVWINNERS} = 0;
	    $divisions{$division}->{MAXWINS} = 0;
            foreach $team (sort {$teams{$b}->{SEASONWINS} <=> $teams{$a}->{SEASONWINS}} (keys %teams)) {
		if ($teams{$team}->{DIVISION} eq $division) {
                    if ($teams{$team}->{SEASONWINS} > $divisions{$division}->{MAXWINS}) {
			$divisions{$division}->{MAXWINS} = $teams{$team}->{SEASONWINS};
		    }
		}
            }
	}  #End of foreach divisions loop
       ###########  Find Division Winners  ###########
	foreach $team (keys %teams) {
            if ($teams{$team}->{SEASONWINS} == $divisions{$teams{$team}->{DIVISION}}->{MAXWINS}) {
		$teams{$team}->{DIVWINNER} = 1;
		$divisions{$teams{$team}->{DIVISION}}->{DIVWINNERS}++;
	    }
	}
	foreach $team (keys %teams) {
            if ($teams{$team}->{DIVWINNER} == 1) {
		$teams{$team}->{DIVTITLES} += 1 / $divisions{$teams{$team}->{DIVISION}}->{DIVWINNERS};
		$teams{$team}->{DIVWINNER}  = 1 / $divisions{$teams{$team}->{DIVISION}}->{DIVWINNERS};
	    }
	}

       ###########  Find Wildcard Winners  ###########
	if ($psFormat == 2) {       #only do this if wildcards enabled

	    foreach $subleague (sort {$a <=> $b} (keys %subLeagues)) {
		$subLeagues{$subleague}->{WC1WINS} = 0;
		$subLeagues{$subleague}->{WC1WINNERS} = 0;
		foreach $team (sort {$teams{$b}->{SEASONWINS} <=> $teams{$a}->{SEASONWINS}} (keys %teams)) {
		    if (($teams{$team}->{SUBLEAGUE} eq $subleague) && ($teams{$team}->{DIVWINNER} != 1)) {
			if ($teams{$team}->{SEASONWINS} > $subLeagues{$subleague}->{WC1WINS}) {
			    $subLeagues{$subleague}->{WC1WINS} = $teams{$team}->{SEASONWINS};
			}
		    }
		}
	    }  #End of Wildcard Subleague loop
          ###########  Find First Wildcard Winners  ###########
	    foreach $team (keys %teams) {
		if (($teams{$team}->{SEASONWINS} == $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC1WINS}) && ($teams{$team}->{DIVWINNER} != 1)) {
		    $teams{$team}->{WC1WINNER} = 1;
		    $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC1WINNERS}++;
		}
	    }
	    foreach $team (keys %teams) {
		if ($teams{$team}->{WC1WINNER} == 1) {
		    $teams{$team}->{WCTITLES}  += 1 / $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC1WINNERS};
		    $teams{$team}->{WC1WINNER}  = 1 / $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC1WINNERS};
		}
	    }

          ###########  Find Second Wildcard Winners  ###########
		foreach $subleague (sort {$a <=> $b} (keys %subLeagues)) {
		  if ($subLeagues{$subleague}->{nDIVS} % 2 == 0) {     # only do second wildcard if even number of divisions
		    $subLeagues{$subleague}->{WC2WINS} = 0;
		    $subLeagues{$subleague}->{WC2WINNERS} = 0;
		    foreach $team (sort {$teams{$b}->{SEASONWINS} <=> $teams{$a}->{SEASONWINS}} (keys %teams)) {
			if (($teams{$team}->{SUBLEAGUE} eq $subleague) && (($teams{$team}->{DIVWINNER} + $teams{$team}->{WC1WINNER}) != 1)) {
			    if ($teams{$team}->{SEASONWINS} > $subLeagues{$subleague}->{WC2WINS}) {
				$subLeagues{$subleague}->{WC2WINS} = $teams{$team}->{SEASONWINS};
			    }
			}
		    }
		    }
		}  #End of Wildcard Subleague loop
             ###########  Find Second Wildcard Winners  ###########
		foreach $team (keys %teams) {
		    if ($subLeagues{$teams{$team}->{SUBLEAGUE}}->{nDIVS} % 2 == 0) {     # only do second wildcard if even number of divisions
		      if (($teams{$team}->{SEASONWINS} == $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC2WINS}) && (($teams{$team}->{DIVWINNER} + $teams{$team}->{WC1WINNER}) != 1)) {
            $teams{$team}->{WC2WINNER} = 1;
            $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC2WINNERS}++;
		      }
		    }
		}
		foreach $team (keys %teams) {
		    if ($subLeagues{$teams{$team}->{SUBLEAGUE}}->{nDIVS} % 2 == 0) {     # only do second wildcard if even number of divisions
		      if ($teams{$team}->{WC2WINNER} == 1) {
            $teams{$team}->{WCTITLES} += 1 / $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC2WINNERS};
            $teams{$team}->{WC2WINNER}  = 1 / $subLeagues{$teams{$team}->{SUBLEAGUE}}->{WC2WINNERS};
		      }
		    }
		}
	  #End of second wildcard block

	}  #End of Wildcard IF block

    }  #End of simulate seasons for loop

    foreach $team (keys %teams) {
	$teams{$team}->{AVGWINS}   = sprintf("%.1f", ($teams{$team}->{TOTALWINS} / $seasons));
	$teams{$team}->{AVGLOSSES} = sprintf("%.1f", ($teams{$team}->{TOTALLOSSES} / $seasons));
	$teams{$team}->{TITLEPCT}  = sprintf("%.1f", (100 * ($teams{$team}->{DIVTITLES} + $teams{$team}->{WCTITLES}) / $seasons));
	if ($psFormat == 2) {
	    $teams{$team}->{DIVTITLES} = sprintf("%.1f", (100 * $teams{$team}->{DIVTITLES} / $seasons));
	}
	else {
	    $teams{$team}->{DIVTITLES} = sprintf("%.1f", $teams{$team}->{DIVTITLES});
	}
	$teams{$team}->{WCTITLES}  = sprintf("%.1f", (100 * $teams{$team}->{WCTITLES} / $seasons));
    }  #End of simulate seasons foreach loop (totals)

    ###########  Print Results to File  ###########
    my $count;
    my $value;
    my $new;
    my $class;
    my $pwins;
    my $plosses;
    my $pdiff;
    my $cols;


    if ($psFormat == 1) {
	$cols = 15;
    }
    else {
	$cols = 16;
    }

    ##### Subleague Table
    foreach $subleague (sort {$a <=> $b} (keys %subLeagues)) {
	print OUT '<table cellspacing="0" cellpadding="1" class="lpo" width="100%" style="margin-bottom:12px;">';
        print OUT "<tr><td class=\"h0\" colspan=\"$cols\">$subLeagues{$subleague}->{NAME}</td></tr>\n";
        print OUT '<tr><td style="padding-top:12px;padding-left:12px;padding-right:12px;">';
    ##### Division Table
        print OUT '<table cellspacing="0" cellpadding="1" class="lpo" width="840" style="margin-bottom:12px;">';
        foreach $division (sort {$a cmp $b} (keys %divisions)) {
            if ($division =~ /^$subleague-/) {
                print OUT "<tr><td class=\"h1_l\" colspan=\"$cols\">$divisions{$division}->{NAME}</td></tr>\n";
            print OUT '<tr><td class="hsc2_l" width="150">Team</td><td class="hsn2">W</td><td class="hsn2">L</td><td class="hsn2">PCT</td><td class="hsn2">RS</td><td class="hsn2">RA</td><td class="hsn2">Pyt.Rec</td><td class="hsn2">Pyth%</td><td class="hsn2">Diff</td><td class="hsn2">AvgW</td><td class="hsn2">AvgL</td><td class="hsn2">MaxW</td><td class="hsn2">MinW</td>';
		if ($psFormat == 2) {
		    print OUT '<td class="hsn2">Div%</td><td class="hsn2">WC%</td>';
		}
		else {
		    print OUT '<td class="hsn2">Div</td>';
		}
		print OUT '<td class="hsn2">PSOdds%</td></tr>';
                $count = 0;
                foreach $team (sort {$teams{$b}->{AVGWINS} <=> $teams{$a}->{AVGWINS}} (keys %teams)) {
                    if ($teams{$team}->{DIVISION} eq $division) {
			if ($count % 2 == 0) {
			    $class = "s1";
			}
			else {
			    $class = "s2";
			}

			print OUT "<tr>";
			print OUT "<td class=\"$class\_l\"><a href=\"$self->{INFO}->{URL}/teams/team\_$team\.html\">$teams{$team}->{CITY} $teams{$team}->{NAME}</a></td>";
			print OUT "<td class=\"$class\">$teams{$team}->{WINS}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{LOSSES}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{WINPCT}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{RUNSFOR}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{RUNSAGAINST}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{PWINS}\-$teams{$team}->{PLOSSES}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{PYTHPCT}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{PDIFF}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{AVGWINS}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{AVGLOSSES}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{MAXWINS}</td>";
			print OUT "<td class=\"$class\">$teams{$team}->{MINWINS}</td>";
			if ($psFormat == 2) {
			    print OUT "<td class=\"$class\">$teams{$team}->{DIVTITLES}</td><td class=\"$class\">$teams{$team}->{WCTITLES}</td>";
			}
			else {
			    print OUT "<td class=\"$class\">$teams{$team}->{DIVTITLES}</td>";
			}
			print OUT "<td class=\"$class\">$teams{$team}->{TITLEPCT}</td>";
			print OUT "</tr>";

                        print OUT "<tr><td>";
                        if ($count == 2) {
                            print OUT "</tr>";
                        }
                        $count++;
                    }
                }
            }
        }
        print OUT "</table></td></tr>\n";
    }

    close (OUT);
}

1
