#!/usr/local/bin/perl -w

package OOTPOU::Ratings;

use strict;
use vars qw(@ISA);
use OOTPOU::Page;
use OOTPOU::Utils;
use OOTPOU::HTMLUtils;
use OOTPOU::DraftUtils;

@ISA = "OOTPOU::Page";

sub printPage {

    my ($self) = @_;

    my @keys;
    my $key;
    my $id;
    my $pos;
    my $role;
    my $name;
    my $value;
    my $output;
    my $i = 0;
    my $submitted = 1;
    my $count = 0;
    my $teamSel;
    my $nTeams;
    my $leagueid = $self->{INFO}->{LEAGUEID};
    my $dir = "OOTPOU/draft/$self->{INFO}->{DATE}->{YEAR}/$leagueid";
    $self->{DRAFTDIR} = $dir;

    OOTPOU::DraftUtils::getDraftInfo($self);

    $self->{PLAYERS} = {};
    $self->{SHADE} = 1;
    $self->{TAL_FACTOR} = 100;
    if ($self->{INFO}->{TALENT} == 0) {
        $self->{TAL_FACTOR} = 8;
    }

    if (!$self->{PARAMS}->{SORT1}) {
        $submitted = 0;
    }

    $self->{PARAMS}->{TEAMSELECT} ||= 0;
    $self->{PARAMS}->{POSITION} ||= 1;
    $self->{PARAMS}->{DOFIELD} ||= 1;
    $self->{PARAMS}->{RATINGS} ||= 2;
    $self->{PARAMS}->{SORT1} ||= 'TAL_VALUE';
    $self->{PARAMS}->{SORT2} ||= 'NAME';
    $self->{PARAMS}->{SORT3} ||= 'NAME';

    if ($self->{PARAMS}->{POSITION} == 2 || $self->{PARAMS}->{POSITION} > 10) {
        $self->{PARAMS}->{DOFIELD} = 2;
    }

    if ($self->{PARAMS}->{DOFIELD} == 1) {
        if ($self->{PARAMS}->{SORT1} eq 'TAL_STUFF' || $self->{PARAMS}->{SORT1} eq 'TAL_CONTROL' ||
            $self->{PARAMS}->{SORT1} eq 'TAL_MOVEMENT' || $self->{PARAMS}->{SORT1} eq 'ENDURANCE') {
            $self->{PARAMS}->{SORT1} = 'NAME';
        }
        if ($self->{PARAMS}->{SORT2} eq 'TAL_STUFF' || $self->{PARAMS}->{SORT2} eq 'TAL_CONTROL' ||
            $self->{PARAMS}->{SORT2} eq 'TAL_MOVEMENT' || $self->{PARAMS}->{SORT2} eq 'ENDURANCE') {
            $self->{PARAMS}->{SORT2} = 'NAME';
        }
        if ($self->{PARAMS}->{SORT3} eq 'TAL_STUFF' || $self->{PARAMS}->{SORT3} eq 'TAL_CONTROL' ||
            $self->{PARAMS}->{SORT3} eq 'TAL_MOVEMENT' || $self->{PARAMS}->{SORT3} eq 'ENDURANCE') {
            $self->{PARAMS}->{SORT3} = 'NAME';
        }
    }
    else {
        if ($self->{PARAMS}->{SORT1} eq 'TAL_CONTACT' || $self->{PARAMS}->{SORT1} eq 'TAL_GAP' ||
            $self->{PARAMS}->{SORT1} eq 'TAL_POWER' || $self->{PARAMS}->{SORT1} eq 'TAL_EYE' ||
            $self->{PARAMS}->{SORT1} eq 'TAL_K') {
            $self->{PARAMS}->{SORT1} = 'NAME';
        }
        if ($self->{PARAMS}->{SORT2} eq 'TAL_CONTACT' || $self->{PARAMS}->{SORT2} eq 'TAL_GAP' ||
            $self->{PARAMS}->{SORT2} eq 'TAL_POWER' || $self->{PARAMS}->{SORT2} eq 'TAL_EYE' ||
            $self->{PARAMS}->{SORT2} eq 'TAL_K') {
            $self->{PARAMS}->{SORT2} = 'NAME';
        }
        if ($self->{PARAMS}->{SORT3} eq 'TAL_CONTACT' || $self->{PARAMS}->{SORT3} eq 'TAL_GAP' ||
            $self->{PARAMS}->{SORT3} eq 'TAL_POWER' || $self->{PARAMS}->{SORT3} eq 'TAL_EYE' ||
            $self->{PARAMS}->{SORT3} eq 'TAL_K') {
            $self->{PARAMS}->{SORT3} = 'NAME';
        }
    }

    if ($self->{PARAMS}->{TEAMSELECT}) {
        $teamSel = $self->{PARAMS}->{TEAMSELECT};
    }
    elsif ($self->{INFO}->{USERTEAMID}) {
        $teamSel = $self->{INFO}->{USERTEAMID};
    }
    else {$teamSel = -1};

    open (IN, "$self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/,0,"[^,]*?"\s*$/ && /^(\d+),"(.*?)","(\w+)","(.*?)",".*?",".*?",\d+,\d+,(\d+),\d+,\d+,\d+,(\d+),/) {
            $self->{TEAMS}->{$1}->{NAME} = "$2 $4";
            $self->{TEAMS}->{$1}->{ABBR} = $3;
            $self->{TEAMS}->{$1}->{LEAGUE} = $5;
            $self->{TEAMS}->{$1}->{LEVEL} = $6;
            $nTeams = $1;
        }
    }
    close(IN);

    if ($teamSel < 1 || $teamSel > $nTeams) {$teamSel = 0};

    open (IN, "$self->{INFO}->{CSV}/leagues.csv");
    while (<IN>) {
        if ( /^(\d+),"(.*?)","(.*?)",\d+,/) {
            $self->{LEAGUES}->{$1}->{NAME} = $2;
            $self->{LEAGUES}->{$1}->{ABBR} = $3;
        }
    }
    close(IN);

    open (IN, "$self->{INFO}->{CSV}/players.csv");

    my $leagueID = $self->{INFO}->{LEAGUEID};
    while (<IN>)
    {
	if (/^(\d+),(\d+),(\d+),"(.*?)","(.*?)","(.*?)",(\d+),"(.*?)",(\d+),(\d+),(\d+),(\d+),(\d+),0,\d+,\d+,\d+,$teamSel,\d+,-?\d+,-?\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,".*?",".*?",\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),(\d+),/)
	{
            $count++;
            $id = $1;
            $self->{PLAYERS}->{$id} = {};
            $self->{PLAYERS}->{$id}->{TEAM}      = $2;
            $self->{PLAYERS}->{$id}->{LEVEL}     = $self->{TEAMS}->{$2}->{LEVEL};
            $self->{PLAYERS}->{$id}->{LEAGUE}    = $3;
            $self->{PLAYERS}->{$id}->{FNAME}     = $4;
            $self->{PLAYERS}->{$id}->{LNAME}     = $5;
            $self->{PLAYERS}->{$id}->{NNAME}     = $6;
            $self->{PLAYERS}->{$id}->{AGE}       = $7;
#           $self->{PLAYERS}->{$id}->{DOB}       = $8;
#           $self->{PLAYERS}->{$id}->{CITY}      = $9;
#           $self->{PLAYERS}->{$id}->{NATION}    = OOTPOU::Utils::convertNat($10);
#           $self->{PLAYERS}->{$id}->{NATION2}   = $11;
#           $self->{PLAYERS}->{$id}->{WEIGHT}    = $12;
#           $self->{PLAYERS}->{$id}->{HEIGHT}    = $13;
            $self->{PLAYERS}->{$id}->{ORG}       = $teamSel;
            $self->{PLAYERS}->{$id}->{POSITION}  = OOTPOU::Utils::convertPos($14);
            $self->{PLAYERS}->{$id}->{ROLE}      = OOTPOU::Utils::convertRole($15);
        }
    }
    close(IN);

#    print 'Count: '.$count;

    open (IN, "$self->{INFO}->{CSV}/players_batting.csv");
    while(<IN>)
    {
        if (/^(\d+)/)
        {
            if ($self->{PLAYERS}->{$1})
            {
                $id = $1;
                if ($self->{PARAMS}->{RATINGS} == 2 && $self->{INFO}->{RATINGS} > 0 && /^\d+,\d+,\d+,(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),/)
                {
                    $self->{PLAYERS}->{$id}->{CONTACT} = OOTPOU::Utils::convertTalent($1, $self->{INFO}->{RATINGS}),
                    $self->{PLAYERS}->{$id}->{GAP}     = OOTPOU::Utils::convertTalent($2, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{EYE}     = OOTPOU::Utils::convertTalent($3, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{K}       = OOTPOU::Utils::convertTalent($4, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{HP}      = OOTPOU::Utils::convertTalent($5, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{POWER}   = OOTPOU::Utils::convertTalent($6, $self->{INFO}->{RATINGS});
                }
                if (/(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),\d+,\d+,\d+,\d+,\d+\s*$/)
                {
                    if ($self->{INFO}->{TALENT} > 0) {
                        $self->{PLAYERS}->{$id}->{TAL_CONTACT}= OOTPOU::Utils::convertTalent($1, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_GAP}    = OOTPOU::Utils::convertTalent($2, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_EYE}    = OOTPOU::Utils::convertTalent($3, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_K}      = OOTPOU::Utils::convertTalent($4, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_HP}     = $5;
                        $self->{PLAYERS}->{$id}->{TAL_POWER}  = OOTPOU::Utils::convertTalent($6, $self->{INFO}->{TALENT});
                    }
                }
                if ($self->{PARAMS}->{pick}) {
                    $self->{PLAYERS}->{$id}->{O_TAL_CONTACT}= OOTPOU::Utils::convertTalent($1, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_GAP}    = OOTPOU::Utils::convertTalent($2, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_EYE}    = OOTPOU::Utils::convertTalent($3, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_K}      = OOTPOU::Utils::convertTalent($4, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_POWER}  = OOTPOU::Utils::convertTalent($6, $self->{TAL_FACTOR});
                }
            }
        }
    }
    close(IN);


    open (IN, "$self->{INFO}->{CSV}/players_pitching.csv");
    while(<IN>)
    {
        if (/^(\d+)/)
        {
            if ($self->{PLAYERS}->{$1})
            {
                $id = $1;
                if ($self->{PARAMS}->{RATINGS} == 2 && $self->{INFO}->{RATINGS} > 0 && /^\d+,\d+,\d+,(\d+),(\d+),(\d+),/)
                {
                    $self->{PLAYERS}->{$id}->{STUFF}    = OOTPOU::Utils::convertTalent($1, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{CONTROL}  = OOTPOU::Utils::convertTalent($2, $self->{INFO}->{RATINGS});
                    $self->{PLAYERS}->{$id}->{MOVEMENT} = OOTPOU::Utils::convertTalent($3, $self->{INFO}->{RATINGS});
                }
                if (/(\d+),(\d+),(\d+),\d+,\d+,\d+,(\d+),\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),(\d+),\d+\s*$/)
                {
                    if ($self->{INFO}->{TALENT} > 0) {
                        $self->{PLAYERS}->{$id}->{TAL_STUFF}  = OOTPOU::Utils::convertTalent($1, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_CONTROL}= OOTPOU::Utils::convertTalent($2, $self->{INFO}->{TALENT});
                        $self->{PLAYERS}->{$id}->{TAL_MOVEMENT}= OOTPOU::Utils::convertTalent($3, $self->{INFO}->{TALENT});
                    }
                    $self->{PLAYERS}->{$id}->{VELOCITY}   = $4;
                    if ($self->{INFO}->{RATINGS} > 0) {
                        $self->{PLAYERS}->{$id}->{ENDURANCE}  = OOTPOU::Utils::convertTalent($5, $self->{INFO}->{RATINGS});
                    }
                    $self->{PLAYERS}->{$id}->{GROUNDBALL} = $6;
                }
                if ($self->{PARAMS}->{pick}) {
                    $self->{PLAYERS}->{$id}->{O_TAL_STUFF}  = OOTPOU::Utils::convertTalent($1, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_CONTROL}  = OOTPOU::Utils::convertTalent($2, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_TAL_MOVEMENT}  = OOTPOU::Utils::convertTalent($3, $self->{TAL_FACTOR});
                    $self->{PLAYERS}->{$id}->{O_ENDURANCE}  = OOTPOU::Utils::convertTalent($5, $self->{TAL_FACTOR});
                }
            }
        }
    }
    close(IN);

    open (IN, "$self->{INFO}->{CSV}/players_value.csv");
    while(<IN>)
    {
        if (/^(\d+)/)
        {
            if ($self->{PLAYERS}->{$1})
            {
                $id = $1;
                if ( /^\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),\d+,\d+,(\d+),/)
                {
                    $self->{PLAYERS}->{$id}->{VALUE}    = OOTPOU::Utils::convertStars($1);
                    $self->{PLAYERS}->{$id}->{TAL_VALUE}  = OOTPOU::Utils::convertStars($2);
                }
            }
        }
    }
    close(IN);

    open (IN, "$self->{INFO}->{CSV}/players_fielding.csv");
    while(<IN>)
    {
        if (/^(\d+)/)
        {
            if ($self->{PLAYERS}->{$1})
            {
                $id = $1;
                if (/(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)\s*$/)
                {
                    if ($self->{INFO}->{RATINGS} > 0) {
                        $self->{PLAYERS}->{$id}->{P}    = OOTPOU::Utils::convertTalent($1, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{C}    = OOTPOU::Utils::convertTalent($2, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{B1}   = OOTPOU::Utils::convertTalent($3, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{B2}   = OOTPOU::Utils::convertTalent($4, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{B3}   = OOTPOU::Utils::convertTalent($5, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{SS}   = OOTPOU::Utils::convertTalent($6, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{LF}   = OOTPOU::Utils::convertTalent($7, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{CF}   = OOTPOU::Utils::convertTalent($8, $self->{INFO}->{RATINGS});
                        $self->{PLAYERS}->{$id}->{RF}   = OOTPOU::Utils::convertTalent($9, $self->{INFO}->{RATINGS});
                    }
                    if ($self->{PARAMS}->{pick}) {
                        $self->{PLAYERS}->{$id}->{O_P}    = OOTPOU::Utils::convertTalent($1, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_C}    = OOTPOU::Utils::convertTalent($2, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_B1}   = OOTPOU::Utils::convertTalent($3, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_B2}   = OOTPOU::Utils::convertTalent($4, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_B3}   = OOTPOU::Utils::convertTalent($5, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_SS}   = OOTPOU::Utils::convertTalent($6, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_LF}   = OOTPOU::Utils::convertTalent($7, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_CF}   = OOTPOU::Utils::convertTalent($8, $self->{TAL_FACTOR});
                        $self->{PLAYERS}->{$id}->{O_RF}   = OOTPOU::Utils::convertTalent($9, $self->{TAL_FACTOR});
                        if ($self->{PLAYERS}->{$id}->{POSITION} eq 'P') {
                            $self->{PLAYERS}->{$id}->{OVERALL} = OOTPOU::DraftUtils::overallPitcher($self, $id);
                        }
                        else {
                            $self->{PLAYERS}->{$id}->{OVERALL} = OOTPOU::DraftUtils::overallBatter($self, $id);
                        }
                    }
                }
            }
        }
    }
    close(IN);

    @keys = keys %{$self->{PLAYERS}};

    @keys = @{$self->sortPlayers('TAL_VALUE', \@keys)};

    @keys = @{$self->sortPlayers($self->{PARAMS}->{SORT3}, \@keys)};
    @keys = @{$self->sortPlayers($self->{PARAMS}->{SORT2}, \@keys)};
    @keys = @{$self->sortPlayers($self->{PARAMS}->{SORT1}, \@keys)};

    print '<form name="draft" method="post" onsubmit="return moveList(document.getElementById(\'draftList\'), document.getElementById(\'dList\'));">';

    $self->printSorts();

    if ($self->{PARAMS}->{DOFIELD} == 1)
    {
        $self->printFieldHead();
    }
    else {
        $self->printPitchHead();
    }

    $count = 1;
    foreach $key (@keys)
    {
        if ($self->{PARAMS}->{POSITION} <= 2 || $self->playsPosition($key, $self->{PARAMS}->{POSITION})) {
            $count += $self->printPlayer($key);
	}
    }
    print "</form></td></tr>";
}

sub sortPlayers {
    my ($self, $sort, $keys) = @_;
    my @keys = @$keys;
    if ($sort)
    {
        if ($sort eq 'NAME')
        {
            @keys = sort{$self->{PLAYERS}->{$a}->{LNAME} cmp $self->{PLAYERS}->{$b}->{LNAME}} @keys;
        }
          if ($sort eq 'AGE' || $sort eq 'LEVEL') {
            @keys = sort{$self->{PLAYERS}->{$a}->{$sort} <=> $self->{PLAYERS}->{$b}->{$sort}} @keys;
        }
        else
        {
            if ($self->{PARAMS}->{RATINGS} == 2 ) {
                $sort =~ s/^TAL_//;
            }
            if ($sort eq 'VELOCITY' || (!($self->{PARAMS}->{RATINGS} == 2 && $self->{INFO}->{RATINGS} == 0) &&
                                        !($self->{PARAMS}->{RATINGS} != 2 && $self->{INFO}->{TALENT} == 0))) {
                @keys = sort{$self->{PLAYERS}->{$b}->{$sort} <=> $self->{PLAYERS}->{$a}->{$sort}} @keys;
            }
        }
    }
    return \@keys;
}

sub playsPosition
{
    my ($self, $player, $pos) = @_;
    if ($pos == 3) {
        return 1 if ($self->{PLAYERS}->{$player}->{C});
    }
    elsif ($pos == 4) {
        return 1 if ($self->{PLAYERS}->{$player}->{B1});
    }
    elsif ($pos == 5) {
        return 1 if ($self->{PLAYERS}->{$player}->{B2});
    }
    elsif ($pos == 6) {
        return 1 if ($self->{PLAYERS}->{$player}->{B3});
    }
    elsif ($pos == 7) {
        return 1 if ($self->{PLAYERS}->{$player}->{SS});
    }
    elsif ($pos == 8) {
        return 1 if ($self->{PLAYERS}->{$player}->{LF});
    }
    elsif ($pos == 9) {
        return 1 if ($self->{PLAYERS}->{$player}->{CF});
    }
    elsif ($pos == 10) {
        return 1 if ($self->{PLAYERS}->{$player}->{RF});
    }
    elsif ($pos == 11) {
        return 1 if ($self->{PLAYERS}->{$player}->{ROLE} eq 'SP');
    }
    elsif ($pos == 12) {
        return 1 if ($self->{PLAYERS}->{$player}->{ROLE} eq 'MR');
    }
    elsif ($pos == 13) {
        return 1 if ($self->{PLAYERS}->{$player}->{ROLE} eq 'CL');
    }
    return 0;
}

sub printSorts
{
    my ($self) = @_;
    my $selected;
    my $pickHTML = '';
    my $listHTML = '';
    if ($self->{ISPICK}) {
        $pickHTML = "&nbsp;<input type=\"button\" onclick=\"pickPlayer();\" value=\"Pick\"></input>\n";
    }

    if (1) {
print <<EOF
<tr>
<td colspan="10" align="left">
<table cellpadding="0" width="100%"><tr>
<td align="left">
<table>
<tr align="left">
<td>
Team&nbsp;<select name="TEAMSELECT">
EOF
}
    print "<option value=\"-1\">Free Agents</option>\n";

    my $teamSel;
    if ($self->{PARAMS}->{TEAMSELECT}) {
        $teamSel = $self->{PARAMS}->{TEAMSELECT};
    }
    elsif ($self->{INFO}->{USERTEAMID}) {
        $teamSel = $self->{INFO}->{USERTEAMID};
    }
    else {$teamSel = 0};

    open (IN, "$self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/,0,"[^,]*?"\s*$/ && /^(\d+),"(.*?)","\w+","(.*?)",".*?",".*?",\d+,\d+,$self->{INFO}->{LEAGUEID},/) {
            $selected = '';
            if ($teamSel == $1) {
              $selected = ' selected';
            }
            print "<option value=\"$1\"$selected>$2 $3</option>\n";
        }
    }
    close(IN);
if (1) {
print <<EOF
</select></td></tr><tr align="right">
<td>
View
<select name="POSITION">
<option value="1">Batters</option>
<option value="2">Pitchers</option>
<option value="3">C</option>
<option value="4">1B</option>
<option value="5">2B</option>
<option value="6">3B</option>
<option value="7">SS</option>
<option value="8">LF</option>
<option value="9">CF</option>
<option value="10">RF</option>
<option value="11">SP</option>
<option value="12">MR</option>
<option value="13">CL</option>
</select>
&nbsp;
Ratings
<select name="RATINGS">
<option value="1">Potential</option>
<option value="2">Actual</option>
</select>
&nbsp;
EOF
}
    if ($self->{PARAMS}->{DOFIELD} == 1) {
print <<EOF
Sort #1
<select name="SORT1">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="LEVEL">League</option>
<option value="AGE">Age</option>
<option value="TAL_CONTACT">Contact</option>
<option value="TAL_GAP">Gap</option>
<option value="TAL_POWER">Power</option>
<option value="TAL_EYE">Eye</option>
<option value="TAL_K">Strikeouts</option>
</select>
&nbsp;
Sort #2
<select name="SORT2">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="LEVEL">League</option>
<option value="AGE">Age</option>
<option value="TAL_CONTACT">Contact</option>
<option value="TAL_GAP">Gap</option>
<option value="TAL_POWER">Power</option>
<option value="TAL_EYE">Eye</option>
<option value="TAL_K">Strikeouts</option>
</select>
&nbsp;
Sort #3
<select name="SORT3">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="LEVEL">League</option>
<option value="AGE">Age</option>
<option value="TAL_CONTACT">Contact</option>
<option value="TAL_GAP">Gap</option>
<option value="TAL_POWER">Power</option>
<option value="TAL_EYE">Eye</option>
<option value="TAL_K">Strikeouts</option>
</select>
</td>
<td>
<input type="submit" value="Go"></input>
</td>
</tr>
</table>
</td>
</tr>
</table>
</td></tr>
EOF
}
else {
print <<EOF2
Sort #1
<select name="SORT1">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="Level">League</option>
<option value="AGE">Age</option>
<option value="TAL_STUFF">Stuff</option>
<option value="TAL_CONTROL">Control</option>
<option value="TAL_MOVEMENT">Movement</option>
<option value="ENDURANCE">Endurance</option>
</select>
&nbsp;
Sort #2
<select name="SORT2">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="Level">League</option>
<option value="AGE">Age</option>
<option value="TAL_STUFF">Stuff</option>
<option value="TAL_CONTROL">Control</option>
<option value="TAL_MOVEMENT">Movement</option>
<option value="ENDURANCE">Endurance</option>
</select>
&nbsp;
Sort #3
<select name="SORT3">
<option value="NAME">Name</option>
<option value="TAL_VALUE">Overall</option>
<option value="LEVEL">League</option>
<option value="AGE">Age</option>
<option value="TAL_STUFF">Stuff</option>
<option value="TAL_CONTROL">Control</option>
<option value="TAL_MOVEMENT">Movement</option>
<option value="ENDURANCE">Endurance</option>
</select>
</td>
<td>
<input type="submit" value="Go"></input>
</td>
</tr>
</table>
</td>
</tr>
</table>
</td></tr>
EOF2
}
    print "<SCRIPT language=\"Javascript\">\n";
    print "function setSorts() {\n";
    print "document.draft.SORT1.value = \"$self->{PARAMS}->{SORT1}\";\n";
    print "document.draft.SORT2.value = \"$self->{PARAMS}->{SORT2}\";\n";
    print "document.draft.SORT3.value = \"$self->{PARAMS}->{SORT3}\";\n";
    print "document.draft.POSITION.value = \"$self->{PARAMS}->{POSITION}\";\n";
    print "document.draft.RATINGS.value = \"$self->{PARAMS}->{RATINGS}\";\n";
    print "}\n";
    print "</SCRIPT>\n";

}

sub printPlayer {
    my ($self, $id) = @_;
    my $printed = 0;

    if ($self->{PARAMS}->{DOFIELD} == 1 && $self->{PLAYERS}->{$id}->{POSITION} ne "P")
    {
        if (($self->{INFO}->{TALENT} == 0 && $self->{PARAMS}->{RATINGS} != 2) || ($self->{INFO}->{RATINGS} == 0 && $self->{PARAMS}->{RATINGS} == 2)) {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{POSITION}</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td></tr>\n";
        }
        elsif ($self->{PARAMS}->{RATINGS} == 2) {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{POSITION}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{CONTACT}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{GAP}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{POWER}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{EYE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{K}</td></tr>\n";
        }
        else {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{POSITION}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_CONTACT}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_GAP}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_POWER}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_EYE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_K}</td></tr>\n";
        }
        $printed = 1;
    }
    elsif ($self->{PARAMS}->{DOFIELD} != 1 && $self->{PLAYERS}->{$id}->{POSITION} eq "P")
    {
        if (($self->{INFO}->{TALENT} == 0 && $self->{PARAMS}->{RATINGS} != 2) || ($self->{INFO}->{RATINGS} == 0 && $self->{PARAMS}->{RATINGS} == 2)) {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{ROLE}</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">-</td><td class=\"s$self->{SHADE}\">".OOTPOU::Utils::convertVelocity($self->{PLAYERS}->{$id}->{VELOCITY})." Mph</td><td class=\"s$self->{SHADE}\">-</td></tr>\n";

        }
        elsif ($self->{PARAMS}->{RATINGS} == 2) {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{ROLE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{STUFF}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{MOVEMENT}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{CONTROL}</td><td class=\"s$self->{SHADE}\">".OOTPOU::Utils::convertVelocity($self->{PLAYERS}->{$id}->{VELOCITY})." Mph</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{ENDURANCE}</td></tr>\n";
        }
        else {
            print "<tr><td class=\"s".$self->{SHADE}."_l\"><a href=\"$self->{INFO}->{URL}/players/player_$id.html\" target=\"draftWindow\">$self->{PLAYERS}->{$id}->{FNAME} $self->{PLAYERS}->{$id}->{LNAME}</a></td><td class=\"s$self->{SHADE}\">$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{ORG}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{LEAGUES}->{$self->{TEAMS}->{$self->{PLAYERS}->{$id}->{TEAM}}->{LEAGUE}}->{ABBR}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{AGE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{ROLE}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_STUFF}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_MOVEMENT}</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{TAL_CONTROL}</td><td class=\"s$self->{SHADE}\">".OOTPOU::Utils::convertVelocity($self->{PLAYERS}->{$id}->{VELOCITY})." Mph</td><td class=\"s$self->{SHADE}\">$self->{PLAYERS}->{$id}->{ENDURANCE}</td></tr>\n";
        }
        $printed = 1;
    }

    if ($printed > 0)
    {
        if ($self->{SHADE} == 1)
        {
            $self->{SHADE} = 2;
        }
        else {
            $self->{SHADE} = 1;
        }
    }
    return $printed;
}

sub printFieldHead {
    my ($self) = @_;
print <<EOF
<tr><td class="h2_l" style="width:120px;">Position Players</td><td class="h2">Team</td><td class="h2">League</td><td class="h2">Age</td><td class="h2">Pos</td><td class="h2">Contact</td><td class="h2">Gap</td><td class="h2">Power</td><td class="h2">Eye</td><td class="h2">Avoid K's</td></tr>
EOF
}

sub printPitchHead {
    my ($self) = @_;
print <<EOF
<tr><td class="h2_l" style="width:120px;">Pitchers</td><td class="h2">Team</td><td class="h2">League</td><td class="h2">Age</td><td class="h2">Role</td><td class="h2">Stuff</td><td class="h2">Movement</td><td class="h2">Control</td><td class="h2">Velocity</td><td class="h2">Endurance</td></tr>
EOF
}
