package OOTPOU::Utils;

sub getDate {
    my ($csvDir, $leagueID) = @_;
    my $date;

    open (INDATE, "$csvDir/leagues.csv");
    while (<INDATE>) {
        if (/^$leagueID,/ && /"(\d+-\d+-\d+)"\s*$/) {
            $date = $1;
            last;
        }
    }
    close (INDATE);
    return $date;
}

sub getLeague{
    my ($csvDir, $leagueID) = @_;

    my $output = `grep -E '^$leagueID,' $csvDir/leagues.csv`;
    if ($output =~ /^\d+,".*?","(.*?)",/) {
        return $1;
    }
}

sub getLeagueName{
    my ($csvDir, $leagueID) = @_;

    my $output = `grep -E '^$leagueID,' $csvDir/leagues.csv`;
    if ($output =~ /^\d+,"(.*?)",/) {
        return $1;
    }
}

sub textDate {
    my ($inDate) = @_;
    my $date;

    if ($inDate =~ /^(\d+)-(\d+)-(\d+)/) {
        if ($2 == 1) {$month='January';}
        elsif ($2 == 2) {$month='February';}
        elsif ($2 == 3) {$month='March';}
        elsif ($2 == 4) {$month='April';}
        elsif ($2 == 5) {$month='May';}
        elsif ($2 == 6) {$month='June';}
        elsif ($2 == 7) {$month='July';}
        elsif ($2 == 8) {$month='August';}
        elsif ($2 == 9) {$month='September';}
        elsif ($2 == 10) {$month='October';}
        elsif ($2 == 11) {$month='November';}
        elsif ($2 == 12) {$month='December';}
        $date = "$month $3, $1";
    }
    return $date;
}

sub convertTime {
    my ($inTime) = @_;
    $inTime =~ /(\d+)(\d\d)$/;
    my $hour = $1;
    my $min = $2;
    my $ampm = 'am';
    if ($hour > 12) {
        $hour -= 12;
        $ampm = 'pm';
    }
    elsif ($hour == 0) {
        $hour = 12;
    }
    return "$hour:$min $ampm";
}

sub min {
    my ($a, $b) = @_;
    if ($a < $b) {
        return $a;
    }
    return $b;
}

sub max {
    my ($a, $b) = @_;
    if ($a > $b) {
        return $a;
    }
    return $b;
}

sub convertTalent {
    my ($inTal, $talScale) = @_;
    my $newTal;
    if ($inTal == 0) {
        return 0;
    }
    if ($talScale == 8) {
        $newTal = int(($inTal+66)/33);
        $newTal = min(8, $newTal);
        $newTal = max(2, $newTal);
    }
    elsif ($talScale == 80) {
        my $factor = int(200/13);
        $newTal = int(($inTal+$factor-5)/$factor);
        $newTal = min(13, $newTal);
        $newTal = max(1, $newTal);
        $newTal = ($newTal*5)+15;
    }
    elsif ($talScale == 0) {
        $newTal = 0;
    }
    else {
        my $factor = 200/$talScale;
        $newTal = int(($inTal+$factor)/$factor);
        $newTal = min($talScale, $newTal);
        $newTal = max(1, $newTal);
    }
    return $newTal;
}

sub convertVelocity {
    my ($inVel) = @_;
    if ($inVel < 9) {
        return ($inVel+79).'-'.($inVel+81);
    }
    elsif ($inVel == 9) {
        return '89-90';
    }
    else {
        return ($inVel+80).'-'.($inVel+82);
    }
}

sub convertRole {
    my ($role) = @_;
    if ($role == 11) {
        return 'SP';
    }
    elsif ($role == 12) {
        return 'MR';
    }
    elsif ($role == 13) {
        return 'CL';
    }
}

sub convertPos {
    my ($inPos) = @_;
    if ($inPos == 1)
    {
        return 'P';
    }
    elsif ($inPos == 2)
    {
        return "C";
    }
    elsif ($inPos == 3)
    {
        return "1B";
    }
    elsif ($inPos == 4)
    {
        return "2B";
    }
    elsif ($inPos == 5)
    {
        return "3B";
    }
    elsif ($inPos == 6)
    {
        return "SS";
    }
    elsif ($inPos == 7)
    {
        return "LF";
    }
    elsif ($inPos == 8)
    {
        return "CF";
    }
    elsif ($inPos == 9)
    {
        return "RF";
    }
}


sub convertStars {
    my ($inStars) = @_;
    if ($inStars < 401)
    {
        return 1;
    }
    elsif ($inStars > 400 && $inStars < 801)
    {
        return 2;
    }
    elsif ($inStars > 800 && $inStars < 1201)
    {
        return 3;
    }
    elsif ($inStars > 1200 && $inStars < 1601)
    {
        return 4;
    }
    else
    {
        return 5;
    }
}

sub setCookie {
    my ($name, $value, $remove) = @_;

    my $expire;
    if ($remove eq 'y') {
        $expire = 'Sat, 09-09-2006 00:00:00 GMT';
    }
    else {
        $expire = 'Sun, 09-09-2012 00:00:00 GMT';
    }
    print "Set-Cookie:";
    print $name."=".$value.";expires=$expire;".
        "path=/;\n";
}

sub getDraftTime
{
    my @timeData = localtime(time);
    my $time = join(' ' , @timeData);
    $time =~ /^\d+ (\d+) (\d+) \d+ \d+ \d+ (\d+) /;
    return "$3-$2-$1";
}

sub getActiveLeagues
{
    my ($self) = @_;
    my %teams;
    my %leagues;
    open (IN, "$self->{INFO}->{CSV}/teams.csv");
    while (<IN>) {
        if (/^(\d+),".*?","\w+",".*?",".*?",".*?",\d+,\d+,(\d+),/) {
            $teams{$1} = $2;
        }
    }
    close (IN);

    open (IN, "$self->{INFO}->{CSV}/human_managers.csv");
    while (<IN>) {
        if (/^\d+,\d+,".*?",".*?",".*?",\d+,".*?",\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,\d+,(\d+),/) {
            if ($teams{$1}) {
                $leagues{$teams{$1}} = {};
            }
        }
    }
    close (IN);

    open (IN, "$self->{INFO}->{CSV}/leagues.csv");
    while (<IN>) {
        if (/^(\d+),"(.*?)","(.*?)",/) {
            if ($leagues{$1}) {
                $leagues{$1}->{NAME}   = $2;
                $leagues{$1}->{ABBREV} = $3;
            }
        }
    }
    close (IN);
    return \%leagues;
}

sub winPct {
    my ($wins, $losses) = @_;
    my $pct;

    if (($wins + $losses) == 0) {
        $pct = 0;
    }
    else {
        $pct = $wins / ($wins + $losses);
    }
    return sprintf("%.3f",$pct);
}

sub PythWinPct {
    my ($rf, $ra, $pythExp) = @_;
    my $pct;

    if (($rf + $ra) == 0) {
        $pct = 0;
    }
    else {
        $pct = ($rf ** $pythExp) / (($rf ** $pythExp) + ($ra ** $pythExp));
    }
    return sprintf("%.3f",$pct);
}

sub PythpatWinPct {
    my ($rf, $ra, $w, $l) = @_;
    my $gp;
    my $exp;
    my $pct;

    $gp = $w + $l;
    if ($gp == 0) {
        $pct = 0;
    }
    else {
        $exp = (($rf + $ra) / $gp) ** 0.287;
	  $pct = ($rf ** $exp) / (($rf ** $exp) + ($ra ** $exp));
    }
    return sprintf("%.3f",$pct);
}

sub pythRec {
    my ($w, $l, $pythPct) = @_;
    my $games;
    my $pythW;
    my $pythL;
    my $diff;

    $games = $w + $l;
    $pythW = sprintf("%.0f",($games * $pythPct));
    $pythL = $games - $pythW;
    $diff  = $w - $pythW;

    return ($pythW, $pythL, $diff);
}

sub log5WinPct {
    my ($homeWpct, $awayWpct) = @_;
    my $team1chance = 0;

    if ($homeWpct + $awayWpct != 0) {
	$team1chance = ($homeWpct - $homeWpct * $awayWpct) / ($homeWpct + $awayWpct- 2 * $homeWpct * $awayWpct);
    }
    return $team1chance;
}


1
